// ---------------------------------------------------------------------------
//
//      The ab-initio based multiscale library
//
//                  S / P H I / n X
//
//      Copyright:  Max-Planck-Institute for Iron Research
//                  40237 Duesseldorf, Germany
//
//      Contact:    https://sxlib.mpie.de
//      Authors:    see sphinx/AUTHORS
//      License:    see sphinx/LICENSE
//
// ---------------------------------------------------------------------------

#ifndef _SX_MATRIXCONV_H_
#define _SX_MATRIXCONV_H_

#include <SxString.h>
#include <SxExt.h>
/** \brief A tool to convert the ascii-matrix formats 
              -.out (supported by the rel1.0)
              -.sx  (to be supported by the rel2.0, readable by parser)
           into each other   
    \ingroup  group_addons
    \author   Lars Ismer, ismer@fhi-berlin.mpg.de */
class SX_EXPORT_EXT SxMatrixConv 
{
   public:
      SxMatrixConv ();
      ~SxMatrixConv ();
  
      /** \brief 
       \loads the matrix from .sx format, 
       returns 1 for success, 0 for failure
      */
      int loadMatrixSx  (const SxString &);
      
      /** \brief 
       \loads the matrix from .out format, 
       returns 1 for success, 0 for failure
      */
      int loadMatrixOut (const SxString &);
     
      /** \brief 
       \saves the matrix to .out format, 
       returns 1 for success, 0 for failure
      */
      int saveMatrixOut (const SxString &);
      
      /** \brief 
       \saves the matrix to .sx format, 
       returns 1 for success, 0 for failure
      */
      int saveMatrixSx (const SxString&);

      SxMatrix<Double> matrix;
      int nDoF;
   
   protected:
};


#endif /* _SX_MATRIXCONV_H_ */
