// ---------------------------------------------------------------------------
//
//      The ab-initio based multiscale library
//
//                  S / P H I / n X
//
//      Copyright:  Max-Planck-Institute for Iron Research
//                  40237 Duesseldorf, Germany
//
//      Contact:    https://sxlib.mpie.de
//      Authors:    see sphinx/AUTHORS
//      License:    see sphinx/LICENSE
//
// ---------------------------------------------------------------------------
#ifndef _SX_ATOMDFT_H_
#define _SX_ATOMDFT_H_

#include <SxDirac.h>

class SxAtomDFT 
{
	public :

		//member functions
		SxAtomDFT(double r0, double rMax, int nIntervals_, int nOrb);
		SxArray<SxDiracVec<Double> > solveKSeqn (const SxDiracVec<Double> &vHartree,
															  const SxDiracVec<Double> &vXc,
                                               const SxArray<SxDiracVec<Int> > &nlPos,
															  double eLower, double eUpper,
															  int z, int nOrb, int nMax, 
                                               int nEl);
		SxDiracVec<Double> getHartree (const SxDiracVec<Double> &trialRho, 
												 int nElectrons,int PoissInitVal); //needs r,nIntervals,dx
		void getXc (const SxDiracVec<Double> &rho);
		double getChemPot (double ekT, double chPot, double deltaChPot, 
								 int nElectrons);

		
		// member variables
		SxDiracVec<Double> r;
		SxDiracVec<Double> eigenKS;
		SxDiracVec<Double> eX, vX, eC, vC;
		SxDiracVec<Double> vEff;
		double r0, rMax, dx;

	private :
		SxDiracVec<Double> getGrid();
		int computeFwdNodes (const SxDiracVec<Double> &vHartree,
									const SxDiracVec<Double> &vXc,									
									double eIn, int z, int l);
		double computeMatchFu (const SxDiracVec<Double> &vHartree,
								     const SxDiracVec<Double> &vXc,									
									  double eIn, int z, int l);
		SxDiracVec<Double> getKSOrbital (const SxDiracVec<Double> &vHartree, 
													const SxDiracVec<Double> &vXc,
													double eIn, int z, int l);
		
		// member variables
		int nIntervals, nNodes;
};

#endif /* _SX_ATOMDFT_H_ */



