// ---------------------------------------------------------------------------
//
//      The ab-initio based multiscale library
//
//                  S / P H I / n X
//
//      Copyright:  Max-Planck-Institute for Iron Research
//                  40237 Duesseldorf, Germany
//
//      Contact:    https://sxlib.mpie.de
//      Authors:    see sphinx/AUTHORS
//      License:    see sphinx/LICENSE
//
// ---------------------------------------------------------------------------

#include <SxCLI.h>
#include <SxGBasis.h>
#include <SxSpectrum.h>

/*
SxVector<Double> peak (double A, double nPeak, double x,
                       const SxVector<Double> &g)
{
   int n = g.getSize ();
   SxVector<Double> res(n);
   for (int i = 0; i < n; ++i)  {
      res(i) = pow(1. + x * (cos(A * g(i)) - 1.), nPeak);
   }
   return res;
}

int main (int argc, char **argv)
{
   SxCLI cli(argc, argv);
   int n = 9*11*81;
   double A = cli.option ("-A", "value", "coupling").toDouble (100);
   double x = cli.option ("-x", "fraction", "active fraction").toDouble (0.05);
   int s0 = cli.option ("-S", "start shell", "starting shell").toInt (1);
   int nShells = cli.option ("-s", "end shell", "end shell").toInt (1);
   cli.finalize ();

   SxFFT1d newFFT(SxFFT::Both, n);

   SxVector<Double> g(n);
   for (int i = 0; i < n; ++i)  {
      g(i) = TWO_PI/n * ((2 * i < n) ? i : (i-n));
   }

   SxVector<Complex16> broadG(n);
   broadG.set (1.);

   for (int s = s0; s <= nShells; s++)  {
      broadG *= peak (A/(s*double(s)*s), s*s, x, g);
   }

   broadG *= exp(-g.sqr () * sqr(5.)/4.);

   SxVector<Complex16> broadR(n);
   newFFT.fftForward (n, broadG.elements, broadR.elements);
   broadR /= n;

   for (int j = -n/2; 2 * j <= n; ++j)  {
      int i = (j < 0 ? j+n : j);
      cout << j << '\t' << broadR(i).re
           << '\t' << broadR(i).im<< endl;

   }

}
*/

#include <SxNArray.h>
#include <SxSphereGrid.h>
#include <SxParser.h>
#include <SxYlm.h>

class Idx2 {
   public:
      int i, j;
      Idx2 (int a, int b) : i(a), j(b) {}
};

Idx2 translate (int i)
{
   // --- Voigt notation: 1,2,3->xx,yy,zz 4->yz, 5->xz, 6->xy
   SX_CHECK_VAR (i < 6 && i >= 0, i);
   if (i < 3) return Idx2(i,i);
   if (i == 3) return Idx2(1,2);
   if (i == 4) return Idx2(0,2);
   /* if (i == 5) */
   return Idx2(0,1);
}

double func2(double g, double a)
{
   double ag = a * g;
   return sqr(g) * exp (-sqr(ag));
}

double sigma (double x)  {
   double res = 1.;
   for (int n = 10; n; n--)  {
      res = 1. + x / n * res;
   }
   return res * exp (-x);
}

/*
double sigma (double x)
{
   return (1. + x*x) * exp (-x*x);
}
*/

int main (int argc, char **argv)
{
   SxCLI cli(argc, argv);
   SxString inFile = cli.option ("-i|--input","file", "input file")
                   .toString ("elastic.sx");
   int gridId = cli.option ("-g","id", "Grid id")
                .toInt (SxSphereGrid::Grid_1454);
   int lmax = cli.option ("-lmax","number", "l max")
                .toInt (8);
   double aLat = cli.option ("--aLat","number","aLat").toDouble (10.);
   double broad = cli.option ("-b|--broad","number","broadening")
                .toDouble (0.);
   cli.last ().defaultValue = "default: automatic";
   cli.newGroup ();
   SxList<double> cElem = cli.option ("--cell", "9 numbers", "cell").toDoubleList ();
   SxCell cell;
   if (cElem.getSize () == 0)  {
      cell = aLat * CellMat(1,0,0, 0,1,0, 0,0,1);
   } else {
      cell = aLat * CellMat(cElem).transpose ();
   }

   cli.finalize ();
   //SxCell cell(CellMat(0,0.5,0.5, 0.5,0,0.5, 0.5,0.5,0));
   //SxCell cell(CellMat(-0.5,0.5,0.5, 0.5,-0.5,0.5, 0.5,0.5,-0.5));


   // -------------------------------------------------
   SxParser parser;
   SxParser::Table table = parser.read (inFile);

   SxMatrix<Double> Cij(6,6);
   SxMatrix3<Double> phi;
   try {
      Cij.set (table->get("Cij")->toList ());
      Cij.reshape (6, 6);
      phi = SxMatrix3<Double> (table->get("phi")->toList ());
   } catch (SxException e)  {
      e.print ();
      SX_EXIT;
   }
   //Cij *= 3.3989316e-05; // GPa -> Hartree/bohr^3
   //Cij *= HA2EV;
   Cij *=  0.0062415097; // GPa-> eV/angstrom^3
   cout << SEPARATOR;
   cout << "| UNITS" << endl;
   cout << "| Elastic constants        : GPa" << endl;
   cout << "| Energies                 : eV" << endl;
   cout << "| Elastic dipole tensor phi: eV" << endl;
   cout << "| Volume                   : angstrom^3" << endl;
   cout << SEPARATOR;

   SxNArray<double, 4> Cijkl(3,3,3,3);

   for (int i = 0; i < 6; i++)  {
      Idx2 ij = translate (i);
      for (int j = 0; j < 6; j++)  {
         Idx2 kl = translate (j);
         Cijkl(ij.i, ij.j, kl.i, kl.j) = Cij(i, j);
         Cijkl(ij.i, ij.j, kl.j, kl.i) = Cij(i, j);
         Cijkl(ij.j, ij.i, kl.i, kl.j) = Cij(i, j);
         Cijkl(ij.j, ij.i, kl.j, kl.i) = Cij(i, j);
      }
   }

   SxSphereGrid::GridType gt = SxSphereGrid::GridType(gridId);
   SxSphereGrid grid(gt);
   cout << "Spherical grid: " << grid.getSize () << endl;
   grid.setupYlm (lmax);

   SxMatrix<Double> K(6,6);
   K.set (0.);
   for (int ig = 0; ig < grid.getSize (); ++ig)  {
      SxVector3<Double> xyz = grid.getXyz (ig);
      SxMatrix3<Double> cil;
      cil.set (0.);
      for (int i = 0; i < 3; ++i)  {
         for (int j = 0; j < 3; ++j)  {
            for (int k = 0; k < 3; ++k)  {
               for (int l = 0; l < 3; ++l)  {
                  cil(i,l) += Cijkl(i,j,k,l) * xyz(j) * xyz(k);
               }
            }
         }
      }
      SxMatrix3<Double> cilInv = cil.inverse ();
      double w = grid.weights(ig);
      for (int ij = 0; ij < 6; ++ij)  {
         Idx2 IJ = translate(ij);
         int i = IJ.i;
         int j = IJ.j;
         for (int kl = 0; kl < 6; ++kl)  {
            Idx2 KL = translate(kl);
            int k = KL.i;
            int l = KL.j;
            for (int a = 0; a < 3; ++a)  {
               for (int b = 0; b < 3; ++b)  {
                  K(ij,kl) += w * xyz(a) * xyz(j) * cilInv(b,i)
                              * Cijkl(a,b,k,l);
               }
            }
         }
      }
   }
   cout << K << endl;
   cout << K.eigenvalues () << endl;
   SxMatrix3<Double> phi2;
   /*
   {
      SxMatrix<Double> phiM(6);
      for (int i = 0; i < 6; ++i)
         phiM(i) = phi(translate(i).i,translate(i).j);
      SxMatrix<Double> phi2M = K.inverse () ^ phiM;
      for (int i = 0; i < 3; ++i)
         phi2(i,i) = phi2M(i);
      phi2(1,2)=phi2(2,1)=phi2M(3);
      phi2(0,2)=phi2(2,0)=phi2M(4);
      phi2(0,1)=phi2(1,0)=phi2M(5);
      cout << phi2 << endl;
   }
   */
   phi2=phi;

   SxVector<Double> eklLm(sqr(lmax+1));
   eklLm.set (0.);
   for (int ig = 0; ig < grid.getSize (); ++ig)  {
      SxVector3<Double> xyz = grid.getXyz (ig);
      SxMatrix3<Double> cil;
      cil.set (0.);
      for (int i = 0; i < 3; ++i)  {
         for (int j = 0; j < 3; ++j)  {
            for (int k = 0; k < 3; ++k)  {
               for (int l = 0; l < 3; ++l)  {
                  cil(i,l) += Cijkl(i,j,k,l) * xyz(j) * xyz(k);
               }
            }
         }
      }
      SxMatrix3<Double> cilInv = cil.inverse ();
      double w = grid.weights(ig);
      Coord phiK = phi2 ^ xyz;
      double ang = phiK ^ cilInv ^ phiK;
      for (int lm = 0; lm < eklLm.getSize (); ++lm)  {
         eklLm(lm) += w * ang * grid.Ylm(ig, lm);
      }
   }

   eklLm *= FOUR_PI; // angular integration
   cout << "Moments of elastic integrand" << endl;
   for (int l = 0; l <= lmax; ++l)  {
      for (int m = -l; m <= l; ++m)  {
         double x = eklLm(SxYlm::combineLm (l, m));
         if (fabs(x) > 1e-10)  {
            cout << "l=" << l << " m=" << m << ": " << x << endl;
         }
      }
   }

   // --- compute energies
   int nlm = sqr(lmax + 1);

   // --- periodic
   if (broad == 0.)  {
      broad = 20. / cbrt (cell.volume);
      cout << "broad = " << broad << endl;
   }
   double eCut = 1.;
   while (sigma(eCut/sqr(broad)) > 1e-16) eCut += 1.;
   cout << "eCut = " << eCut << endl;
   SxCell recCell = cell.getReciprocalCell ();
   SxMesh3D mesh = SxGBasis::getMeshSize (eCut, cell);
   int meshSize = mesh.product (), ng = 0;
   SxSpectrum data;
   data.nPerPeak=15;
   data.set (-10. * broad, sqrt(eCut), broad, nlm);
   SxVector<Double> Ylm(nlm);
   SxVector<Double> latticeMoments(nlm);
   latticeMoments.set (0.);
   double pref0 = 0.;
   for (int iMesh = 1; iMesh < meshSize; ++iMesh)  {
      Coord g = recCell.relToCar(mesh.getMeshVec(iMesh, SxMesh3D::Origin));
      if (g.normSqr () > eCut) continue;
      SxYlm::getYlmArray (lmax, g, &Ylm);
      //Ylm /= cell.volume;
      //data.addPeak (g.norm (), Ylm);
      latticeMoments += sigma (g.normSqr () / sqr(broad)) * Ylm;
      ++ng;

      // --- direct summation of elastic energy
      SxVector3<Double> xyz = g / g.norm ();
      SxMatrix3<Double> cil;
      cil.set (0.);
      for (int i = 0; i < 3; ++i)  {
         for (int j = 0; j < 3; ++j)  {
            for (int k = 0; k < 3; ++k)  {
               for (int l = 0; l < 3; ++l)  {
                  cil(i,l) += Cijkl(i,j,k,l) * xyz(j) * xyz(k);
               }
            }
         }
      }
      SxMatrix3<Double> cilInv = cil.inverse ();
      Coord phiK = phi2 ^ xyz;
      double ang = phiK ^ cilInv ^ phiK;
      ang -= 1./sqrt(FOUR_PI) * eklLm(0);
      pref0 += sigma (g.normSqr () / sqr(broad)) * ang;
   }
   cout << "ng=" << ng << endl;
   cout << "min sigma: " << sigma(eCut / sqr(broad)) << endl;
   //data.compute ();

   //FILE *fp = fopen ("Ylm.dat", "w");
   //data.fprint (fp);
   //fclose (fp);
   cout << "Moments of lattice summation" << endl;
   for (int l = 0; l <= lmax; ++l)  {
      for (int m = -l; m <= l; ++m)  {
         latticeMoments(SxYlm::combineLm (l, m)) *= SxYlm::getYlmNormFactor (l, m);
         double x = latticeMoments(SxYlm::combineLm (l, m));
         if (fabs(x) > 1e-10)  {
            cout << "l=" << l << " m=" << m << ": " << x << endl;
         }
      }
   }
   //cout << latticeMoments << endl;

   latticeMoments(0) = 0.; // remove elastic self-interaction
   cout << "pref direct= " << (-0.5 * pref0) << endl;
   cout << "pref lm= " << (-0.5 * dot(eklLm,latticeMoments)) << endl;

   double B0 = 0.;
   for (int i = 0; i < 3; ++i)
     for (int j = 0; j < 3; ++j)
        B0 += Cij(i,j);
   B0 /= 9.;
   cout << "B0 = " << (B0/ 0.0062415097) << " GPa" << endl;

   cout << SEPARATOR;
   double Vf = (phi(0,0) + phi(1,1) + phi(2,2))/(3. * B0);
   cout << "| Formation volume = " << Vf << endl;

   //double prefRel   = -0.5 * dot(eklLm,latticeMoments);
   double prefRel   = -0.5 * pref0;
   double prefConst = prefRel + 0.5 * B0 * Vf * Vf;
   cout << "| pref (relaxed vol): " << prefRel << endl;
   cout << "| pref (const vol): " << prefConst << endl;
   cout << SEPARATOR;
}
