// ---------------------------------------------------------------------------
//
//      The ab-initio based multiscale library
//
//                  S / P H I / n X
//
//      Copyright:  Max-Planck-Institute for Iron Research
//                  40237 Duesseldorf, Germany
//
//      Contact:    https://sxlib.mpie.de
//      Authors:    see sphinx/AUTHORS
//      License:    see sphinx/LICENSE
//
// ---------------------------------------------------------------------------

#include <SxRho.h>
#include <SxCLI.h>

/**
  @ingroup Add-Ons
  @author  Alexey Dick, dick@fhi-berlin.mpg.de
**/

int main (int argc, char **argv)  { 
   
   // --- parse command line arguments
   SxCLI cli(argc, argv);
   cli.preUsageMessage 
      = SxString(" Tool creates Hesse matrix file that contains"
                 " summation of two input Hesse matrices").wrap ();
   SxString inFile1
      = cli.option ("--i1","file","input Hesse matrix file.")
             .toString ();
    SxString inFile2
      = cli.option ("--i2","file","input Hesse matrix file.")
             .toString ();
   int nDOF
      = cli.option ("--dof","integer","degrees of freedom.")
             .toInt ();
   SxString outFile
            = cli.option ("-o","file","output Hessematrix file")
             .toString   ("Hesse.sx");
   cli.finalize ();
   cout << endl;
   
   // --- read matrices
   SxMatrix<Double> hesse1 (nDOF, nDOF), hesse2 (nDOF, nDOF);
   SxBinIO io;
   io.open (inFile1, SxBinIO::ASCII_READ_ONLY);
   io.read ("dummy", &hesse1, nDOF, nDOF);
   io.close ();
   io.open (inFile2, SxBinIO::ASCII_READ_ONLY);
   io.read ("dummy", &hesse2, nDOF, nDOF);
   io.close ();

   // add matrices
   SxMatrix<Double> result = hesse1 + hesse2;

   // --- write matrix 
   io.open (outFile, SxBinIO::ASCII_WRITE_ONLY);
   io.write ("dummy", result, "dummy", "dummy");
   io.close();
   
   return 0;
}
