// ---------------------------------------------------------------------------
//
//      The ab-initio based multiscale library
//
//                  S / P H I / n X
//
//      Copyright:  Max-Planck-Institute for Iron Research
//                  40237 Duesseldorf, Germany
//
//      Contact:    https://sxlib.mpie.de
//      Authors:    see sphinx/AUTHORS
//      License:    see sphinx/LICENSE
//
// ---------------------------------------------------------------------------
#include <SxRho.h>
#include <SxCLI.h>

int main(int argc, char **argv)
{
   initSPHInXMath ();

   SxCLI cli(argc, argv);
   cli.authors = "C. Freysoldt";

   cli.preUsageMessage 
      = "This add-on creates a supercell mesh file.";

   SxString meshIn 
      = cli.option ("-i|--input","file","file").toString ("rho.sxb");

   SxString outFile
      = cli.option ("-o|--output","file","file").toString ();

   // ---  definition
   SxMatrix3<Int> repMatrix;

   int simpleRep = cli.newGroup ("simple repetition");

   SxVector3<Int> diag (cli.option ("-r|--repeat","3 numbers",
                                     "repetition along current cell vectors")
                            .toIntList3 ("x,"));

   /*
   int trueRep = cli.newGroup ("3dimensional repetition");
   cli.excludeGroup (simpleRep);
   SxVector3<Int> col1 (cli.option ("--a1","vector",
                                     "new a1 in relative coordinates")
                         .toIntList3 ()),
                  col2 (cli.option ("--a2","vector",
                                     "new a2 in relative coordinates")
                         .toIntList3 ()),
                  col3 (cli.option ("--a3","vector",
                                     "new a3 in relative coordinates")
                         .toIntList3 ());
   */
   if (!cli.error)  {
      if (cli.groupAvailable (simpleRep))  {
         if (diag.product () <= 0)  {
            cout << "Illegal repetition factors " << diag << endl;
            cli.setError ();
         }
      /*
      } else if (cli.groupAvailable (trueRep))  {
         repMatrix = SxMatrix3<Int> (col1, col2, col3).transpose ();
         cout << repMatrix << endl;
         if (repMatrix.determinant () == 0)  {
            cout << "Illegal repetition matrix with zero determinant." << endl;
            cli.setError ();
         }
      */
      } else {
         cout << "no repetition given!" << endl;
         cli.setError ();
      }
   }

   cli.finalize ();

   SxRBasis R;
   SxBinIO io;
   SxMesh3D mesh;
   try  {
      io.open (meshIn, SxBinIO::BINARY_READ_ONLY);
      io.read ("dim", &mesh);
      R.cell.read (io);
      R.fft3d.mesh = mesh;
      R.fft3d.meshSize = mesh.product ();
   } catch (SxException e)  {
      e.print ();
      SX_EXIT;
   }
   SxRho rho(io, &R);
   io.close ();

   int nMeshes = int(rho.rhoR.getSize ());

   // --- expand mesh data
   SxMesh3D newMesh = mesh * diag;
   for (int iMesh = 0; iMesh < nMeshes; ++iMesh)  {
      SxMeshR newRho (newMesh.product ());
      for (int x = 0; x < newMesh(0); ++x)  {
         for (int y = 0; y < newMesh(0); ++y)  {
            for (int z = 0; z < newMesh(0); ++z)  {
               newRho (newMesh.getMeshIdx (x, y, z, SxMesh3D::Positive))
                  = rho(iMesh)(mesh.getMeshIdx (x, y, z, SxMesh3D::Unknown));
            }
         }
      }
      rho(iMesh)=newRho;
   }

   R.cell = SxCell (R.cell.basis(0) * diag(0),
                    R.cell.basis(1) * diag(1),
                    R.cell.basis(2) * diag(2));
   R.fft3d.mesh = newMesh;
   R.fft3d.meshSize = newMesh.product ();

   rho.writeRho (outFile);

   return 0;
}

