// ---------------------------------------------------------------------------
//
//      The ab-initio based multiscale library
//
//                  S / P H I / n X
//
//      Copyright:  Max-Planck-Institute for Iron Research
//                  40237 Duesseldorf, Germany
//
//      Contact:    https://sxlib.mpie.de
//      Authors:    see sphinx/AUTHORS
//      License:    see sphinx/LICENSE
//
// ---------------------------------------------------------------------------

// --- Including header-files
#include <SxFile.h>

// --- Constructors
// Constructor
SxFile::SxFile (const SxString &path_)
   : SxFileInfo (path_)
{
   // empty
}

SxFile::SxFile (const SxFileInfo &in)
   : SxFileInfo (in)
{
   // empty
}

// Destructor
SxFile::~SxFile ()
{
   // empty
}

// --- Methods
SxString SxFile::getIncludeName (const SxString &path_)
{
   if (path_.getSize () == 0) return getName ();
   return getIncludeName(path_.tokenize (':'));
}

SxString SxFile::getIncludeName (const SxList<SxString> &path_)
{

   SxList<SxString> partialPath = getAbsPath ().tokenize ('/');
   SxList<SxString>::ConstIterator pathIt, pPathIt = partialPath.fromLast ();

   SxString fileName;
   bool found = false;
   for (pPathIt = partialPath.fromLast (); pPathIt.valid (); --pPathIt)  {
      if (fileName.getSize () > 0)
         fileName = *pPathIt + "/" + fileName;
      else
         fileName = *pPathIt;
      for (pathIt = path_.begin (); pathIt != path_.end (); ++pathIt)  {
         if (SxFile(*pathIt + "/" + fileName).exists ())  {
            found = true;
            break;
         }
      }
      if (found) break;
   }
   if (!found) return SxString ();
   return fileName;
}

bool SxFile::exists () const
{
   // Checking whether the absolute path leads to an existing file
   return isFile () || isSymLink ();
}
