// ---------------------------------------------------------------------------
//                                          
//      The ab-initio based multiscale library 
//     
//                  S / P H I / n X
//                  http://www.sphinxlib.de
//     
//      Contact:    Sixten Boeck, boeck@mpie.de
//                  Algorithm Design and Modeling Group
//                  Computational Materials Design
//                  Max-Planck-Institute for Iron Research
//                  40237 Duesseldorf, Germany
//     
//      Authors:    see src/AUTHORS
//     
// ---------------------------------------------------------------------------

template<class T>
size_t sxHash (const T &in)
{
   return SxHashFunction::hash (in);
}

template<class T, class E>
SxGraph<T,E>::SxGraph ()
   : SxThis<SxGraph<T,E> > (),
     nodes(0, 64),
     firstElement(-1),
     lastElement(-1),
     firstFree(-1),
     sortEdges(false),
     nNodes(0),
     nEdges(0),
     hashSize(0),
     hashUsed(0),
     hashBound(0)
{
   // empty
}

template<class T, class E>
SxGraph<T,E>::~SxGraph ()
{
   // empty
}

//void changeBlocksize (ssize_t blockSize);
//
//template<class T, class E>
//void SxGraph<T,E>::changeBlocksize (ssize_t blockSize_)
//{
//   // slow
//   SxHashedArray<Node> tmp(0, blockSize_);
//   tmp = nodes;
//   nodes = tmp;
//}

template<class T, class E>
void SxGraph<T,E>::setSortEdges (bool b)
{
   sortEdges = b;
}

//template<class T, class E>
//typename SxGraph<T,E>::Node &SxGraph<T,E>::operator() (const T &elem)
//{
//   ssize_t idxHash;
//   if (!hashContains (elem, &idxHash))  {
//      int idx = 0;
//      if (firstFree >= 0)  {
//         idx = firstFree;
//         firstFree = nodes(idx).next;
//      }  else  {
//         idx = static_cast<int>(nodes.size);
//         nodes.resize (nodes.size + 1);
//      }
//      Node &entry   = nodes(idx);
//      entry.element = elem;
//      entry.prev    = lastElement;
//      entry.next    = -1;
//      
//      if (lastElement >= 0)  nodes(lastElement).next = idx;
//      lastElement = idx;
//      if (firstElement < 0)  firstElement = idx;
//      nNodes++;
//   
//      hashTable.elements[idxHash] = lastElement;
//   }
//   
//   return nodes(hashTable.elements[idxHash]);
//}
//
//template<class T, class E>
//typename SxGraph<T,E>::Node &SxGraph<T,E>::getNode (int idx)
//{
//   return nodes(idx);
//}

template<class T, class E>
void SxGraph<T,E>::append (const T &elem)
{
   ssize_t idxHash;
   if (!hashContains (elem, &idxHash))  {
      int idx = 0;
      if (firstFree >= 0)  {
         idx = firstFree;
         firstFree = nodes(idx).next;
      }  else  {
         idx = static_cast<int>(nodes.size);
         nodes.resize (nodes.size + 1);
      }
      Node &entry   = nodes(idx);
      entry.element = elem;
      entry.prev    = lastElement;
      entry.next    = -1;
      
      if (lastElement >= 0)  nodes(lastElement).next = idx;
      lastElement = idx;
      if (firstElement < 0)  firstElement = idx;
      nNodes++;
   
      hashTable.elements[idxHash] = lastElement;
   }
}

template<class T, class E>
void SxGraph<T,E>::appendEdge (const T &from,
                                   const T &to,
                                   const E &elem,
                                   bool    overwrite)
{
   int idxFrom = findPos (from);
   SX_CHECK_VAR (idxFrom >= 0, idxFrom);
   int idxTo   = findPos (to);
   SX_CHECK_VAR (idxTo >= 0, idxTo);
   
   ssize_t idx = -1;
   
   if (overwrite)  {
      idx = findEdgeOut (idxFrom, idxTo);
      if (idx >= 0)  {
         // --- overwrite existing edge
         nodes(idxFrom).out.elements[idx].value = elem;
         return;
      }
   }
   if (idx < 0)  {
      if (sortEdges)  {
         // --- append new edge
         nodes(idxFrom).out.binaryInsert (SxPair<int,E>(idxTo,elem));
         if (nodes(idxTo).in.binarySearch (idxFrom) < 0)  {
            // --- one pointer to parent is enough, incoming edges are unique
            nodes(idxTo).in.binaryInsert (idxFrom);
         }
      }  else  {
         nodes(idxFrom).out.append (SxPair<int,E>(idxTo,elem));
         if (nodes(idxTo).in.findPos (idxFrom) < 0)  {
            nodes(idxTo).in.append (idxFrom);
         }      
      }
      nEdges++;
   }
}

template<class T, class E>
void SxGraph<T,E>::prependEdge (const T &from,
                                    const T &to,
                                    const E &elem,
                                    bool    overwrite)
{
   int idxFrom = findPos (from);
   SX_CHECK_VAR (idxFrom >= 0, idxFrom);
   int idxTo   = findPos (to);
   SX_CHECK_VAR (idxTo >= 0, idxTo);
   
   ssize_t idx = -1;
   
   if (overwrite)  {
      idx = findEdgeOut (idxFrom, idxTo);
      if (idx >= 0)  {
         // --- overwrite existing edge
         nodes(idxFrom).out.elements[idx].value = elem;
         return;
      }
   }
   if (idx < 0)  {
      if (sortEdges)  {
         // --- append new edge
         nodes(idxFrom).out.binaryInsert (SxPair<int,E>(idxTo,elem));
         if (nodes(idxTo).in.binarySearch (idxFrom) < 0)  {
            // --- one pointer to parent is enough, incoming edges are unique
            nodes(idxTo).in.binaryInsert (idxFrom);
         }
      }  else  {
         nodes(idxFrom).out.prepend (SxPair<int,E>(idxTo,elem));
         if (nodes(idxTo).in.findPos (idxFrom) < 0)  {
            nodes(idxTo).in.prepend (idxFrom);
         }      
      }
      nEdges++;
   }
}

template<class T, class E>
SxGraph<T,E>&
SxGraph<T,E>::append (const SxGraph<T,E> &in)
{
   // --- nodes
   int ptr = in.firstElement;
   while (ptr >= 0)  {
      append (in.nodes(ptr).element);
      ptr = in.nodes(ptr).next;
   }
   // --- edges
   ptr = in.firstElement;
   while (ptr >= 0)  {
      const SxArrayList<SxPair<int,E>,int> &edgesOut = in.nodes(ptr).out;
      for (int i=0; i < edgesOut.size; i++)  {
         appendEdge (in.nodes(ptr).element,
                     in.nodes(edgesOut.elements[i].key).element,
                     edgesOut.elements[i].value, false);
      }
      ptr = in.nodes(ptr).next;
   }
      
   return *this;
}

template<class T, class E>
void SxGraph<T,E>::removeElement (const T &elem)
{
   int ptr = findPos (elem);
   SX_CHECK_VAR (ptr >= 0, ptr);
   
   // --- remove edges
   Node &node = nodes(ptr);
   const SxArrayList<SxPair<int,E>,int> &edgesOut = node.out;
   for (int i=0; i < edgesOut.size; i++)  {
      disconnectIn (ptr, edgesOut.elements[i].key);
      --nEdges;
   }
   const SxArrayList<int,int> &edgesIn = node.in;
   for (int i=0; i < edgesIn.size; i++)  {
      disconnectOut (edgesIn.elements[i], ptr);
   }
   
   // --- remove node
   int prevPtr = node.prev;
   int nextPtr = node.next;
   nodes(ptr) = Node(); // destroy

   if (prevPtr >= 0) nodes(prevPtr).next = nextPtr;
   else              firstElement = nextPtr;
   if (nextPtr >= 0) nodes(nextPtr).prev = prevPtr;
   else              lastElement = prevPtr;
   
   node.next = firstFree;
   node.prev = -1;
   firstFree = ptr;
   --nNodes;
}

template<class T, class E>
void SxGraph<T,E>::removeEdge (const T &from, const T &to)
{
   int idxFrom = findPos (from);
   SX_CHECK_VAR (idxFrom >= 0, idxFrom);
   int idxTo   = findPos (to);
   SX_CHECK_VAR (idxTo >= 0, idxTo);
   
   disconnectOut (idxFrom, idxTo);
   disconnectIn  (idxFrom, idxTo);
}

template<class T, class E>
bool SxGraph<T,E>::hasOutEdge (int idxFrom, int idxTo) const
{
   return findEdgeOut (idxFrom, idxTo) >= 0;
}

template<class T, class E>
ssize_t SxGraph<T,E>::findEdgeOut (int idxFrom, int idxTo) const
{
   const SxArrayList<SxPair<int,E>,int> &edgesOut = nodes(idxFrom).out;
   if (sortEdges)  {
      ssize_t l = 0;
      ssize_t r = edgesOut.size - 1;
      while (r >= l)  {
         const ssize_t m = (l + r) / 2;
         if (idxTo == edgesOut.elements[m].key) return m;
         if (idxTo <  edgesOut.elements[m].key) r = m - 1;
         else                                   l = m + 1;
      }
   }  else  {
      const ssize_t n = edgesOut.size;
      for (ssize_t i=0; i < n; ++i)  {
         if (idxTo == edgesOut.elements[i].key) return i;
      }
   }
   return -1;
}

template<class T, class E>
ssize_t SxGraph<T,E>::findEdgeIn (int idxFrom, int idxTo) const
{
   if (sortEdges)  {
      return nodes(idxFrom).in.binarySearch (idxTo);
   }  else  {
      return  nodes(idxFrom).in.findPos (idxTo);
   }
}

template<class T, class E>
void SxGraph<T,E>::disconnectOut (int idxFrom, int idxTo)
{
   // --- remove all outgoing
   SxArrayList<SxPair<int,E>,int> &edgesOut = nodes(idxFrom).out;
   if (sortEdges)  {
      ssize_t idx1 = -1;
      ssize_t idx2 = -1;
      ssize_t l = 0;
      ssize_t r = edgesOut.size - 1;
      while (r >= l)  {
         const ssize_t m = (l + r) / 2;
         if (idxTo == edgesOut.elements[m].key) {
            idx1 = idx2 = m;
            break;
         }
         if (idxTo < edgesOut.elements[m].key) r = m - 1;
         else                                  l = m + 1;
      }
      if (idx1 >= 0)  {
         // --- connected directly
         r = edgesOut.size - 1;
         while (idx1 > 0 && edgesOut.elements[idx1 - 1].key == idxTo)  --idx1;
         while (idx2 < r && edgesOut.elements[idx2 + 1].key == idxTo)  ++idx2;
         ssize_t removeNEdges = (idx2 - idx1) + 1;
         edgesOut.remove (idx1, removeNEdges);
         
         nEdges -= removeNEdges;
      }
   }  else  {
      ssize_t removeNEdges = 0;
      ssize_t idx = -1;
      const ssize_t n = edgesOut.size;
      for (ssize_t i=0; i < n; ++i)  {
         if (idxTo != edgesOut.elements[i].key)  {
            if (idx >= 0)  {
               // --- shift to empty place from removed edges
               edgesOut.elements[idx] = edgesOut.elements[i];
               idx++;
            }
         }  else  {
            removeNEdges++;
            if (idx < 0) idx = i; // insert place
         }
      }
      if (removeNEdges > 0)  {
         nEdges -= removeNEdges;
         edgesOut.remove (n - removeNEdges, removeNEdges);
      }
   }
}

template<class T, class E>
void SxGraph<T,E>::disconnectIn (int idxFrom, int idxTo)
{
   // --- remove incomming
   SxArrayList<int,int> &edgesIn = nodes(idxTo).in;
   ssize_t idx = -1;
   if (sortEdges)  {
      idx = edgesIn.binarySearch (idxFrom);
   }  else  {
      idx = edgesIn.findPos (idxFrom);
   }
   if (idx >= 0)  {
      // --- connected directly
      edgesIn.remove (idx);
   }
}

template<class T, class E>
void SxGraph<T,E>::removeAll ()
{
   nodes.removeAll ();
   
   firstElement = -1;
   lastElement = -1;
   firstFree = -1;
   nNodes = 0;
   nEdges = 0;
   
   hashTable.removeAll ();
   hashSize = 0;
   hashUsed = 0;
   hashBound = 0;
}

template<class T, class E>
bool SxGraph<T,E>::contains (const T &elem) const
{
   return findPos (elem) >= 0;
}

template<class T, class E>
int SxGraph<T,E>::findPath (const T         &from,
                                const T         &to,
                                int             maxPathLen,
                                const Selection &selection) const
{
   int idxFrom = findPos (from);
   if (idxFrom < 0 || (selection.getPtr () && !selection->contains (idxFrom)))
      return 0;
   int idxTo = findPos (to);
   if (idxTo < 0 || (selection.getPtr () && !selection->contains (idxTo)))
      return 0;
   
   if (findEdgeOut (idxFrom, idxTo) >= 0)  {
      return 1;
   }
   
   int pathLen = 0;
   SxSet<int> visited;
   SxList<SxPair<int,int> > queue;
   
   visited << idxFrom;
   nextLevel (idxFrom, pathLen, maxPathLen, selection, &queue, &visited);
   
   while (queue.getSize () > 0)  {
      int ptr = queue.first().key;
      pathLen = queue.first().value;
      queue.removeFirst ();

      ssize_t idx = findEdgeOut (ptr, idxTo);
      if (idx >= 0)  {
         return pathLen + 1;
      }
      nextLevel (ptr, pathLen, maxPathLen, selection, &queue, &visited);
   }

   return 0;
}

template<class T, class E>
SxList<T> SxGraph<T,E>::getPath (const T         &from,
                                     const T         &to,
                                     int             maxPathLen,
                                     const Selection &selection) const
{
   SxList<T> res;

   int idxFrom = findPos (from);
   if (idxFrom < 0 || (selection.getPtr () && !selection->contains (idxFrom)))
      return res;
   int idxTo = findPos (to);
   if (idxTo < 0 || (selection.getPtr () && !selection->contains (idxTo)))
      return res;
   
   if (findEdgeOut (idxFrom, idxTo) >= 0)  {
      res.append (nodes(idxFrom).element);
      res.append (nodes(idxTo).element);
   }  else  {
      int pathLen = 0;
      SxSet<int> visited;
      SxList<SxPair<int,int> > queue;
      
      visited << idxFrom;
      nextLevel (idxFrom, pathLen, maxPathLen, selection, &queue, &visited);
      
      while (queue.getSize () > 0)  {
         int ptr = queue.first().key;
         pathLen = queue.first().value;
         queue.removeFirst ();

         ssize_t idx = findEdgeOut (ptr, idxTo);
         if (idx >= 0)  {
            // --- reverse iteration to get the elements in path
            res.append (nodes(idxTo).element);
            visited.removeElement (idxTo);
            while (ptr != idxFrom)  {
               res.prepend (nodes(ptr).element);
               visited.removeElement (ptr);
               // --- parent
               const SxArrayList<int,int> &edgesIn = nodes(ptr).in;
               ssize_t n = edgesIn.size;
               bool found = false;
               for (ssize_t i=0; i < n; ++i)  {
                  if (visited.contains (edgesIn(i)))  {
                     ptr = edgesIn(i);
                     found = true;
                     break;
                  }
               }
               SX_CHECK (found); // inf loop
            }
            res.prepend (nodes(ptr).element);
            break;
         }
         nextLevel (ptr, pathLen, maxPathLen, selection, &queue, &visited);
      }
   }
   return res;
}

template<class T, class E>
void SxGraph<T,E>::nextLevel (int                      ptr,
                                  int                      pathLen,
                                  int                      maxPathLen,
                                  const Selection          &selection,
                                  SxList<SxPair<int,int> > *queue,
                                  SxSet<int>               *visited) const
{
   if (maxPathLen == 0 || pathLen + 1 < maxPathLen)  {
      SX_CHECK (ptr >= 0);
      SX_CHECK (queue);
      SX_CHECK (visited);

      // --- search in children
      const SxArrayList<SxPair<int,E>,int> &edgesOut = nodes(ptr).out;
      if (selection.getPtr ())  {
         for (int i=0; i < edgesOut.size; i++)  {
            int key = edgesOut.elements[i].key;
            if (selection->contains (key) && !visited->contains (key))  {
               queue->append (SxPair<int,int>(key, pathLen + 1));
               visited->append (key);
            }
         }
      }  else  {
         for (int i=0; i < edgesOut.size; i++)  {
            int key = edgesOut.elements[i].key;
            if (!visited->contains (key))  {
               queue->append (SxPair<int,int>(key, pathLen + 1));
               visited->append (key);
            }
         }
      }
   }
}

template<class T, class E>
SxList<T> SxGraph<T,E>::topsort (const Selection &selection_)
   const
{
   SxList<T> res;
   SxList<int> s;
   int ptr = 0;
   
   // --- number of incoming edges for each vertex and start nodes
   ssize_t len = nodes.mapSize * (nodes.lastBlockIdx + 1);
   SxArray<int> edges(len);
   edges.set (0);
   if (selection_.getPtr ())  {
      typename SxList<int>::ConstIterator it;
      for (it = selection_->begin(); it != selection_->end(); ++it)  {
         int nSelected = 0;
         const SxArrayList<int,int> &edgesIn = nodes(*it).in;
         for (int i=0; i < edgesIn.size; i++)  {
            if (selection_->contains (edgesIn(i)))  {
               nSelected++;
            }
         }
         edges(*it) = nSelected;
         if (nSelected < 1)  s << (*it);
      }
   }  else  {
      ptr = firstElement;
      while (ptr >= 0)  {
         edges(ptr) = nodes(ptr).in.size;
         if (nodes(ptr).in.size < 1)  s << ptr;
         ptr = nodes(ptr).next;
      }
   }

   // --- remove incoming edges and collect new start nodes
   while (s.getSize () > 0)  {
      ptr = s.first ();
      s.removeFirst ();
      res << nodes(ptr).element;
      
      const SxArrayList<SxPair<int,E>,int> &edgesOut = nodes(ptr).out;
      for (int i=0; i < edgesOut.size; i++)  {
         int idxTo = edgesOut(i).key;
         if (edges(idxTo) > 0)  {
            edges(idxTo)--;
            if (edges(idxTo) < 1)  s << idxTo;
         }
      }
   }
   
   // --- check cycles
   for (ssize_t i = 0; i < len; ++i)  {
      if (edges(i) > 0)  {
         SX_THROW ("graph has a cycle");
      }
   }            
   return res;
}

// Enumerating Circuits and Loops in Graphs with Self-Arcs and Multiple-Arcs
// K.A. Hawick and H.A. James
// Proc. 2008 Int. Conf. on Foundations of Computer Science (FCS’08), p 14-20
// 2008
template<class T, class E>
SxList<SxList<int> > SxGraph<T,E>::getCyclesIdx (
   const Selection &selection_) const
{
   SxList<SxList<int> > circuits;
   SxArray<bool> visited;
   SxArray<bool> blocked;
   SxArray<SxArrayList<int> > B; // recursive block/unblock
   SxList<int> stack;               // curent circuit
   
   ssize_t len = nodes.mapSize * (nodes.lastBlockIdx + 1);
   blocked.resize (len);
   B.resize (len);
   visited.resize (len);
   
   // --- test circuit in each node
   if (selection_.getPtr ())  {
      if (len > 0)  visited.set (true);
      typename SxList<int>::ConstIterator it;
      for (it = selection_->begin(); it != selection_->end(); ++it)  {
         visited(*it) = false;
      }
      for (it = selection_->begin(); it != selection_->end(); ++it)  {
         blocked.set (false);
         for (ssize_t i=0; i < len; i++)  {
            if (B(i).getSize () > 0) B(i).removeAll ();
         }
         circuit (*it, *it, stack, visited, blocked, B, &circuits);
         visited(*it) = true;
      }
   }  else  {
      if (len > 0)  visited.set (false);
      int ptr = firstElement;
      while (ptr >= 0)  {
         blocked.set (false);
         for (ssize_t i=0; i < len; i++)  {
            if (B(i).getSize () > 0) B(i).removeAll ();
         }
         circuit (ptr, ptr, stack, visited, blocked, B, &circuits);
         visited(ptr) = true;
         ptr = nodes(ptr).next;
      }
   }
   return circuits;
}

template<class T, class E>
SxArray<SxArray<T> > SxGraph<T,E>::getCycles (
   const Selection &selection_) const
{
   SxList<SxList<int> > circuits = getCyclesIdx (selection_);
      
   // --- convert graph indices to elements in found circuits
   SxArray<SxArray<T> > res;
   res.resize (circuits.getSize ());
   SxList<SxList<int> >::ConstIterator it;
   ssize_t i = 0;
   for (it = circuits.begin(); it != circuits.end(); ++it)  {
      res(i).resize (it->getSize ());
      SxList<int>::ConstIterator it2;
      ssize_t j = 0;
      for (it2 = it->begin(); it2 != it->end(); ++it2)  {
         res(i)(j) = nodes(*it2).element;
         j++;
      }
      i++;
   }
   return res;
}

// getCycles
template<class T, class E>
bool SxGraph<T,E>::circuit (int                        idx,
                                int                        start,
                                SxList<int>                &stack,
                                SxArray<bool>              &visited,
                                SxArray<bool>              &blocked,
                                SxArray<SxArrayList<int> > &B,
                                SxList<SxList<int> >       *circuits) const
{
   SX_CHECK (circuits);
   
   bool res = false;
   stack.append (idx);
   blocked(idx) = true;
   
   const SxArrayList<int,int> &edgesOut = nodes(idx).out;
   for (int i=0; i < edgesOut.size; i++)  {
      int to = edgesOut(i);
      if (!visited(to))  {
         if (to == start)  {
            // --- new circuit
            circuits->append (stack);
            res = true;
         }  else if (!blocked(to))  {
            if (circuit (to, start, stack, visited, blocked, B, circuits))  {
               res = true;
            }
         }
      }
   }
   
   if (res)  {
      unblock (idx, blocked, B);
   }  else  {
      for (int i=0; i < edgesOut.size; i++)  {
         int to = edgesOut(i);
         if (!visited(to) && !B(to).contains (idx))  {
            B(to).append (idx);
         }
      }
   }
   
   stack.removeLast ();
   return res;
}

// getCycles
template<class T, class E>
void SxGraph<T,E>::unblock (int                        idx,
                                SxArray<bool>              &blocked,
                                SxArray<SxArrayList<int> > &B) const
{
   blocked(idx) = false;
   
   for (ssize_t i=0; i < B(idx).size; i++)  {
      int e = B(idx)(i);
      B(idx).remove (e);
      i--;
      if (blocked(e))  {
         unblock (e, blocked, B);
      }
   }
}

template<class T, class E>
void SxGraph<T,E>::print (
   typename SxGraph<T,E>::Iterator it,
   std::ostream                      &s,
   const SxString                    &opt) const
{
   SxUniqueList<int> visited;
   typename SxGraph<T,E>::Iterator itEnd;
   for (; it != itEnd; ++it)  {
      visited << it.getIdx ();
   }
   
   s << "digraph G {" << endl;
   if (opt != "")  {
      s << opt << endl;
   }
   
   typename SxList<int>::ConstIterator itIdx;
   for (itIdx = visited.begin(); itIdx != visited.end(); ++itIdx)  {
      const Node *node = &nodes(*itIdx);
      if (node->out.size < 1 && node->in.size < 1)  {
         s << "   \"" << node->element << "\"" << endl;
      }  else  {
         const SxArrayList<SxPair<int,E>,int> &edgesOut = node->out;
         for (int i=0; i < edgesOut.size; i++)  {
            if (visited.contains (edgesOut(i).key))  {
               s << "   \"" << node->element << "\" -> \""
                 << nodes(edgesOut(i).key).element << "\" [label=\""
                 << edgesOut(i).value << "\"]" << endl;
            }
         }
      }
   }
   s << "}" << endl;
}

template<class T, class E>
void SxGraph<T,E>::printDebug () const
{
   size_t nBytesNodes = 0;
   size_t nBytesEdges = 0;
   int ptr = firstElement;
   while (ptr >= 0)  {
      nBytesNodes += sizeof(nodes(ptr))
                   + ::getNBytes (nodes(ptr).element)
                         - sizeof(nodes(ptr).element);
      const SxArrayList<SxPair<int,E>,int> &edgesOut = nodes(ptr).out;
      for (int i=0; i < edgesOut.size; i++)  {
         nBytesEdges += ::getNBytes (edgesOut.elements[i]);
      }
      nBytesEdges += sizeof(int) * nodes(ptr).in.size;
      ptr = nodes(ptr).next;
   }

                       
   cout << "nodes : " << nNodes << " ... " << nBytesNodes << " bytes" << endl;
   cout << "edges : " << nEdges << " ... " << nBytesEdges << " bytes" << endl;
   cout << "mem   : " << getNBytes() << " bytes" << endl;
   sxprintf ("ptr   : %p\n", (void*)nodes.map);
}

template<class T, class E>
int SxGraph<T,E>::findPos (const T &elem) const
{
   ssize_t idx = hashFindPos (elem);
   return (idx >= 0) ? hashTable.elements[idx] : -1;
}

template<class T, class E>
ssize_t SxGraph<T,E>::hashFindPos (const T &elem) const
{
   if (hashSize > 0)  {
      size_t h = sxHash (elem);
      ssize_t idx = h % hashTable.size;
      ssize_t step = h % (hashTable.size - 1) + 1;
      ssize_t lastIdx = idx;
      
      do {
         if (hashTable.elements[idx] == -1)  {
            return -1;
         }  else if (hashTable.elements[idx] != -2 &&
                     nodes(hashTable.elements[idx]).element == elem)
         {
            return idx;
         }
         idx = (idx + step) % hashTable.size;
      } while (idx != lastIdx);
   }
   return -1;
}

template<class T, class E>
bool SxGraph<T,E>::hashContains (const T &elem_, ssize_t *result_)
{
   SX_CHECK (result_);

   if (hashUsed >= hashBound)  {
      if (hashSize * 2 < hashTable.size)  {
         hashResize (hashTable.size - 1);
      }  else  {
         hashResize (hashTable.size + 1);
      }
   }
   
   size_t h = sxHash (elem_);
   ssize_t idx = h % hashTable.size;
   ssize_t step = h % (hashTable.size - 1) + 1;
   ssize_t lastIdx = idx;
   ssize_t replaceIdx = hashTable.size;
   
   do {
      if (hashTable.elements[idx] == -1)  {
         // --- insert new element
         if (replaceIdx != hashTable.size)  {
            idx = replaceIdx;
         }  else  {
            hashUsed++;
         }
         hashSize++;
         *result_ = idx;
         return false;
      }  else if (hashTable.elements[idx] == -2)  {
         replaceIdx = idx;
      }  else if (nodes(hashTable.elements[idx]).element == elem_)  {
         *result_ = idx;
         return true;
      }
      idx = (idx + step) % hashTable.size;
   } while (idx != lastIdx);
   
   throw SxException ("Can not insert.", __FILE__, __LINE__);
}

template<class T, class E>
void SxGraph<T,E>::hashResize (ssize_t newSize_)
{
   if (newSize_ >= 1610612741)  {
      throw SxException ("HashTable hashSize is too big", __FILE__, __LINE__);
   }
   
   // --- http://planetmath.org/encyclopedia/GoodHashTablePrimes.html
   ssize_t prime[30] = {0, 3, 11, 23, 53, 97, 193, 389, 769, 1543, 3079, 6151,
                        12289, 24593, 49157, 98317, 196613, 393241, 786433,
                        1572869, 3145739, 6291469, 12582917, 25165843, 50331653,
                        100663319, 201326611, 402653189, 805306457, 1610612741};
   ssize_t idx = 0;
   while (prime[idx] <= newSize_)  {
      idx++;
   }
   newSize_ = prime[idx];
   ssize_t newBound = (ssize_t)((double)newSize_ * 0.77 + 0.5);
   ssize_t n = hashTable.size;
   ssize_t i;
   
   if (hashSize < newBound)  {
      SxArray<int> newTable(newSize_);
      if (newSize_ > 0)  {
         newTable.set (-1);
      }
      
      for (i = 0; i < n; i++)  {
         if (hashTable.elements[i] >= 0)  {
            size_t h = sxHash (nodes(hashTable.elements[i]).element);
            idx = h % newSize_;
            ssize_t step = h % (newSize_ - 1) + 1;
            ssize_t lastIdx = idx;
            
            while (newTable.elements[idx] >= 0)  {
               idx = (idx + step) % newSize_;
               if (idx == lastIdx)  {
                  SX_EXIT;
               }
            }
            newTable.elements[idx] = hashTable.elements[i];
         }
      }
            
      hashUsed = hashSize;
      hashBound = newBound;
      hashTable.resize (newSize_, false);
      size_t len = sizeof(int) * (size_t)hashTable.size;
      ::memcpy (hashTable.elements, newTable.elements, len);
   }
}

template<class T, class E>
size_t SxGraph<T,E>::getNBytes () const
{
   size_t nBytes = sizeof(*this)
                 + ::getNBytes (hashTable) - sizeof(hashTable);
   
   for (ssize_t i = 0; i < nodes.mapSize; i++)  {
      // --- size of one segment
      nBytes += sizeof (Node *); // sizeof map[i]
      const Node *block = nodes.map[i];
      SX_CHECK (block);
      for (ssize_t j=0; j <= nodes.lastBlockIdx; j++)  {
         nBytes += sizeof(block[j])
                 + ::getNBytes (block[j].element)
                       - sizeof(block[j].element)
                 + ::getNBytes (block[j].in)
                       - sizeof(block[j].in)
                 + ::getNBytes (block[j].out)
                       - sizeof(block[j].out);
      }
   }
              
   return nBytes;
}

template<class T, class E>
size_t getNBytes (const SxGraph<T,E> &in)
{
   return in.getNBytes ();
}

// ----------------------------------------------------------------------------
template<class T, class E>
SxGraph<T,E>::Iterator::Iterator ()
   : graph(NULL),
     node(NULL),
     idx(-1),
     follow(0),
     pathLen(0),
     selectedSizeIn(-1),
     selectedSizeOut(-1)
{
   // empty
}

template<class T, class E>
SxGraph<T,E>::Iterator::Iterator (SxGraph<T,E>  *graph_,
                                      int               idx_,
                                      int               follow_,
                                      const Selection   &selection_)
   : graph(graph_),
     selection(selection_),
     node(NULL),
     idx(idx_),
     follow(follow_),
     pathLen(0),
     selectedSizeIn(-1),
     selectedSizeOut(-1)
{
   SX_CHECK (graph);
   
   if (follow)  {
      if (idx >= 0)  {
         queue << SxPair<int,int>(idx, pathLen);
         visited << idx;
         next ();
      }
   }  else  {
      if (selection.getPtr ())  {
         selectionIt = selection->begin ();
         if (selectionIt.valid ()) ++selectionIt;
      }
      node = (idx >= 0) ? &graph->nodes(idx) : NULL;
   }
}

template<class T, class E>
bool SxGraph<T,E>::Iterator::operator!= (
   const typename SxGraph<T,E>::Iterator &it)
{
   return idx != it.idx;
}

template<class T, class E>
typename SxGraph<T,E>::Iterator &
SxGraph<T,E>::Iterator::operator++ ()
{
   
   if (follow)  {
      next ();
   }  else if (selection.getPtr ()) {
      if (selectionIt.valid ())  {
         idx = *selectionIt;
         node = &graph->nodes(idx);
         ++selectionIt;
      }  else  {
         idx = -1;
         node = NULL;
      }
   }  else  {
      idx = node->next;
      node = (idx >= 0) ? &graph->nodes(idx) : NULL;
   }
   // --- reset neighbors in selection
   selectedSizeIn = -1;
   selectedSizeOut = -1;
   return *this;
}

template<class T, class E>
void SxGraph<T,E>::Iterator::next ()
{
   if (queue.getSize () < 1)  {
      idx  = -1;
      node = NULL;
      return;
   }
   
   idx     = queue.first().key;
   if (idx < 0)  {
      node = NULL;
      return;
   }
   node    = &graph->nodes(idx);
   pathLen = queue.first().value;
   queue.removeFirst ();
   
   if (follow & 1)  {
      // --- follow outgouing
      const SxArrayList<SxPair<int,E>,int> &edges = node->out;
      ssize_t n = edges.size;
      if (selection.getPtr ())  {
         for (ssize_t i=0; i < n; ++i)  {
            if (  !visited.contains (edges(i).key)
                && selection->contains (edges(i).key))
            {
               queue << SxPair<int,int>(edges(i).key, pathLen + 1);
               visited << edges(i).key;
            }
         }
      }  else  {
         for (ssize_t i=0; i < n; ++i)  {
            if (!visited.contains (edges(i).key))  {
               queue << SxPair<int,int>(edges(i).key, pathLen + 1);
               visited << edges(i).key;
            }
         }
      }
   }
   if (follow & 2)  {
      // --- follow incoming
      const SxArrayList<int,int> &edges = node->in;
      ssize_t n = edges.size;
      if (selection.getPtr ())  {
         for (ssize_t i=0; i < n; ++i)  {
            if (  !visited.contains (edges(i))
                && selection->contains (edges(i)))
            {
               queue << SxPair<int,int>(edges(i), pathLen + 1);
               visited << edges(i);
            }
         }
      }  else  {
         for (ssize_t i=0; i < n; ++i)  {
            if (!visited.contains (edges(i)))  {
               queue << SxPair<int,int>(edges(i), pathLen + 1);
               visited << edges(i);
            }
         }
      }
   }
}

//template<class T, class E>
//bool SxGraph<T,E>::Iterator::setParent ()
//{
//   SX_CHECK (node);
//   const SxArrayList<int,int> &edges = node->in;
//   ssize_t n = edges.size;
//   for (ssize_t i=0; i < n; ++i)  {
//      if (visited.contains (edges(i)))  {
//         idx  = edges(i);
//         node = &graph->nodes(idx);
//         return true;
//      }
//   }
//   return false;
//}

template<class T, class E>
T &SxGraph<T,E>::Iterator::operator* ()
{
   SX_CHECK (node);
   return node->element; 
}

template<class T, class E>
T *SxGraph<T,E>::Iterator::operator-> ()
{
   SX_CHECK (node);
   return &node->element; 
}

template<class T, class E>
T &SxGraph<T,E>::Iterator::in (ssize_t idx_)
{
   SX_CHECK (node);
   if (selection.getPtr ())  {
      if (selectedSizeIn < 0)  {
         // --- build cache of selected neighbors
         selectedSizeIn = 0;
         const SxArrayList<int,int> &edgesIn = node->in;
         ssize_t n = edgesIn.size;
         for (ssize_t i=0; i < n; ++i)  {
            if (selection->contains (edgesIn(i)))  {
               selectedSizeIn++;
            }
         }
         selectedIn.resize (selectedSizeIn);
         selectedSizeIn = 0;
         for (ssize_t i=0; i < n; ++i)  {
            if (selection->contains (edgesIn(i)))  {
               selectedIn(selectedSizeIn) = i;
               selectedSizeIn++;
            }
         }
      }
      return graph->nodes(node->out(selectedIn(idx_))).element;
   }
   return graph->nodes(node->in(idx_)).element;
}

template<class T, class E>
T &SxGraph<T,E>::Iterator::out (ssize_t idx_)
{
   SX_CHECK (node);
   if (selection.getPtr ())  {
      if (selectedSizeOut < 0)  {
         // --- build cache of selected neighbors
         selectedSizeOut = 0;
         const SxArrayList<SxPair<int,E>,int> &edgesOut = node->out;
         ssize_t n = edgesOut.size;
         for (ssize_t i=0; i < n; ++i)  {
            if (selection->contains (edgesOut(i).key))  {
               selectedSizeOut++;
            }
         }
         selectedOut.resize (selectedSizeOut);
         selectedSizeOut = 0;
         for (ssize_t i=0; i < n; ++i)  {
            if (selection->contains (edgesOut(i).key))  {
               selectedOut(selectedSizeOut) = i;
               selectedSizeOut++;
            }
         }
      }
      return graph->nodes(node->out(selectedOut(idx_)).key).element;
   }
   return graph->nodes(node->out(idx_).key).element;
}

template<class T, class E>
ssize_t SxGraph<T,E>::Iterator::getSizeIn () const
{
   SX_CHECK (node);
   if (selection.getPtr ())  {
      // --- count selected neighbors
      ssize_t nSelected = 0;
      const SxArrayList<int,int> &edgesIn = node->in;
      ssize_t n = edgesIn.size;
      for (ssize_t i=0; i < n; ++i)  {
         if (selection->contains (edgesIn(i)))  {
            nSelected++;
         }
      }
      return nSelected;
   }
   return node->in.size;
}

template<class T, class E>
ssize_t SxGraph<T,E>::Iterator::getSizeOut () const
{
   SX_CHECK (node);
   if (selection.getPtr ())  {
      // --- count selected neighbors
      ssize_t nSelected = 0;
      const SxArrayList<SxPair<int,E>,int> &edgesOut = node->out;
      ssize_t n = edgesOut.size;
      for (ssize_t i=0; i < n; ++i)  {
         if (selection->contains (edgesOut(i).key))  {
            nSelected++;
         }
      }
      return nSelected;
   }
   return node->out.size;
}

template<class T, class E>
ssize_t SxGraph<T,E>::Iterator::findEdgeOut (int idxTo) const
{
   SX_CHECK (idx >= 0);
   return graph->findEdgeOut (idx, idxTo);
}

template<class T, class E>
ssize_t SxGraph<T,E>::Iterator::findEdgeIn (int idxTo) const
{
   SX_CHECK (idx >= 0);
   return graph->findEdgeIn (idx, idxTo);
}

template<class T, class E>
typename SxGraph<T,E>::Iterator
SxGraph<T,E>::begin (const Selection &selection_)
{
   int idx = -1;
   if (selection_.getPtr ())  {
      if (selection_->getSize () > 0) idx = selection_->first ();
   }  else  {
      idx = firstElement;
   }
   return typename SxGraph<T,E>::Iterator (this, idx, 0, selection_);
}

template<class T, class E>
typename SxGraph<T,E>::Iterator
SxGraph<T,E>::begin (const T         &elem_,
                         int             follow_,
                         const Selection &selection_)
{
   int idx = findPos (elem_);
   if (selection_.getPtr () && !selection_->contains (idx)) idx = -1;
   return typename SxGraph<T,E>::Iterator (this, idx, follow_, selection_);
}

template<class T, class E>
typename SxGraph<T,E>::Iterator SxGraph<T,E>::end ()
{
   return typename SxGraph<T,E>::Iterator ();
}
// ----------------------------------------------------------------------------
template<class T, class E>
SxGraph<T,E>::EdgeIterator::EdgeIterator ()
   : graph(NULL),
     node(NULL),
     edgeIdx(-1),
     singleNode(false)
{
   // empty
}

template<class T, class E>
SxGraph<T,E>::EdgeIterator::EdgeIterator (
   SxGraph<T,E>  *graph_,
   int               idx_,
   const Selection   &selection_)
   : graph(graph_),
     selection(selection_),
     node(NULL),
     edgeIdx(0),
     singleNode(false)
{
   SX_CHECK (graph);

   if (idx_ >= 0)  {
      node = &graph->nodes(idx_);
      singleNode = true;
      if (node->out.size < 1)  {
         node = NULL;
      }  else if (    selection.getPtr ()
                  && !selection->contains (node->out(edgeIdx).key))
      {
         findNextEdge ();
      }
   }  else  {
      if (selection.getPtr ())  {
         selectionIt = selection->begin (); 
         findNextEdge ();
      }  else  {
         if (graph->firstElement >= 0)  {
            node = &graph->nodes(graph->firstElement);
         }
         // --- find first node with some edges
         while (node && node->out.size < 1)  {
            node = (node->next >= 0) ? &graph->nodes(node->next) : NULL;
         }
      }
   }
}

template<class T, class E>
bool SxGraph<T,E>::EdgeIterator::operator!= (
   const typename SxGraph<T,E>::EdgeIterator &in)
{
   return node != in.node;
}

template<class T, class E>
typename SxGraph<T,E>::EdgeIterator &
SxGraph<T,E>::EdgeIterator::operator++ ()
{
   SX_CHECK (node);

   if (selection.getPtr ())  {
      findNextEdge ();
   }  else  {
      edgeIdx++;
      if (edgeIdx >= node->out.size)  {
         if (singleNode)  {
            node = NULL;
         }  else  {
            // --- find next node with some edges
            edgeIdx = 0;
            do {
               node = (node->next >= 0) ? &graph->nodes(node->next) : NULL;
            }  while (node && node->out.size < 1);
         }
      }
   }
   return *this;
}

template<class T, class E>
void SxGraph<T,E>::EdgeIterator::findNextEdge ()
{
   do {
      if (node && edgeIdx < static_cast<ssize_t>(node->out.size - 1))  {
         edgeIdx++;
      }  else  {
         edgeIdx = 0;
         node = NULL;
         while (selectionIt.valid ())  {
            node = &graph->nodes(*selectionIt);
            ++selectionIt;
            if (node->out.size > 0) break;
            else                    node = NULL;
         }
      }
   }  while (node && !selection->contains (node->out(edgeIdx).key));
}

template<class T, class E>
E &SxGraph<T,E>::EdgeIterator::operator* ()
{
   SX_CHECK (node);
   return node->out(edgeIdx).value;
}

template<class T, class E>
E *SxGraph<T,E>::EdgeIterator::operator-> ()
{
   SX_CHECK (node);
   return &(node->out(edgeIdx).value);
}

template<class T, class E>
T &SxGraph<T,E>::EdgeIterator::from ()
{
   SX_CHECK (node);
   return node->element;
}

template<class T, class E>
T &SxGraph<T,E>::EdgeIterator::to ()
{
   SX_CHECK (node);
   return graph->nodes(node->out(edgeIdx).key).element;
}

template<class T, class E>
typename SxGraph<T,E>::EdgeIterator
SxGraph<T,E>::beginEdge (const Selection &selection_)
{
   return typename SxGraph<T,E>::EdgeIterator (this, -1, selection_);
}

//template<class T, class E>
//typename SxGraph<T,E>::EdgeIterator
//SxGraph<T,E>::beginEdge (const T &elem_, SxUniqueList<int> *selection_)
//{
//   int idx = findPos (elem_);
//   if (idx < 0 || (selection_ && !selection_->contains (idx)))
//      return typename SxGraph<T,E>::EdgeIterator ();
//   else
//      return typename SxGraph<T,E>::EdgeIterator (this, idx, selection_);
//}

template<class T, class E>
typename SxGraph<T,E>::EdgeIterator SxGraph<T,E>::endEdge ()
{
   return typename SxGraph<T,E>::EdgeIterator ();
}
// ----------------------------------------------------------------------------
template<class T>
SxGraph<T,SxNull>::SxGraph ()
   : SxThis<SxGraph<T,SxNull> > (),
     nodes(0, 64),
     firstElement(-1),
     lastElement(-1),
     firstFree(-1),
     sortEdges(false),
     nNodes(0),
     nEdges(0),
     hashSize(0),
     hashUsed(0),
     hashBound(0)
{
   // empty
}

template<class T>
SxGraph<T,SxNull>::~SxGraph ()
{
   // empty
}

//void changeBlocksize (ssize_t blockSize);
//
//template<class T>
//void SxGraph<T,SxNull>::changeBlocksize (ssize_t blockSize_)
//{
//   // slow
//   SxHashedArray<Node> tmp(0, blockSize_);
//   tmp = nodes;
//   nodes = tmp;
//}

template<class T>
void SxGraph<T,SxNull>::setSortEdges (bool b)
{
   sortEdges = b;
}

template<class T>
void SxGraph<T,SxNull>::append (const T &elem)
{
   ssize_t idxHash;
   if (!hashContains (elem, &idxHash))  {
      int idx = 0;
      if (firstFree >= 0)  {
         idx = firstFree;
         firstFree = nodes(idx).next;
      }  else  {
         idx = static_cast<int>(nodes.size);
         nodes.resize (nodes.size + 1);
      }
      Node &entry   = nodes(idx);
      entry.element = elem;
      entry.prev    = lastElement;
      entry.next    = -1;
      
      if (lastElement >= 0)  nodes(lastElement).next = idx;
      lastElement = idx;
      if (firstElement < 0)  firstElement = idx;
      nNodes++;
   
      hashTable.elements[idxHash] = lastElement;
   }
}

template<class T>
void SxGraph<T,SxNull>::appendEdge (const T &from, const T &to)
{
   int idxFrom = findPos (from);
   SX_CHECK_VAR (idxFrom >= 0, idxFrom);
   int idxTo   = findPos (to);
   SX_CHECK_VAR (idxTo >= 0, idxTo);
   
   if (hasOutEdge (idxFrom, idxTo)) return;

   if (sortEdges)  {
      nodes(idxFrom).out.binaryInsert (idxTo);
      nodes(idxTo).in.binaryInsert (idxFrom);
   }  else  {
      nodes(idxFrom).out.append (idxTo);
      nodes(idxTo).in.append (idxFrom);   
   }
   nEdges++;
}

template<class T>
void SxGraph<T>::prependEdge (const T &from, const T &to)
{
   int idxFrom = findPos (from);
   SX_CHECK_VAR (idxFrom >= 0, idxFrom);
   int idxTo   = findPos (to);
   SX_CHECK_VAR (idxTo >= 0, idxTo);
   
   if (hasOutEdge (idxFrom, idxTo)) return;

   if (sortEdges)  {
      nodes(idxFrom).out.binaryInsert (idxTo);
      nodes(idxTo).in.binaryInsert (idxFrom);
   }  else  {
      nodes(idxFrom).out.prepend (idxTo);
      nodes(idxTo).in.prepend (idxFrom);   
   }
   nEdges++;
}

template<class T>
SxGraph<T,SxNull>&
SxGraph<T,SxNull>::append (const SxGraph<T,SxNull> &in)
{
   // --- nodes
   int ptr = in.firstElement;
   while (ptr >= 0)  {
      append (in.nodes(ptr).element);
      ptr = in.nodes(ptr).next;
   }
   // --- edges
   ptr = in.firstElement;
   while (ptr >= 0)  {
      const SxArrayList<int,int> &edgesOut = in.nodes(ptr).out;
      for (int i=0; i < edgesOut.size; i++)  {
         appendEdge (in.nodes(ptr).element,
                     in.nodes(edgesOut.elements[i]).element);
      }
      ptr = in.nodes(ptr).next;
   }
   
   return *this;
}

template<class T>
void SxGraph<T,SxNull>::removeNode (int nodeIdx)
{
   // --- remove edges
   Node &node = nodes(nodeIdx);
   const SxArrayList<int,int> &edgesOut = node.out;
   for (int i=0; i < edgesOut.size; i++)  {
      disconnectIn (nodeIdx, edgesOut.elements[i]);
      --nEdges;
   }
   const SxArrayList<int,int> &edgesIn = node.in;
   for (int i=0; i < edgesIn.size; i++)  {
      disconnectOut (edgesIn.elements[i], nodeIdx);
   }
   
   // --- remove node
   int prevPtr = node.prev;
   int nextPtr = node.next;
   nodes(nodeIdx) = Node(); // destroy

   if (prevPtr >= 0) nodes(prevPtr).next = nextPtr;
   else              firstElement = nextPtr;
   if (nextPtr >= 0) nodes(nextPtr).prev = prevPtr;
   else              lastElement = prevPtr;
   
   node.next = firstFree;
   node.prev = -1;
   firstFree = nodeIdx;
   --nNodes;
}

template<class T>
void SxGraph<T,SxNull>::removeElement (const T &elem)
{
   int ptr = findPos (elem);
   SX_CHECK_VAR (ptr >= 0, ptr);
   removeNode (ptr);
}

template<class T>
void SxGraph<T,SxNull>::removeEdge (const T &from, const T &to)
{
   int idxFrom = findPos (from);
   SX_CHECK_VAR (idxFrom >= 0, idxFrom);
   int idxTo   = findPos (to);
   SX_CHECK_VAR (idxTo >= 0, idxTo);
   
   disconnectOut (idxFrom, idxTo);
   disconnectIn  (idxFrom, idxTo);
}

template<class T>
bool SxGraph<T,SxNull>::hasOutEdge (int idxFrom, int idxTo) const
{
   return findEdgeOut (idxFrom, idxTo) >= 0;
}

template<class T>
ssize_t SxGraph<T,SxNull>::findEdgeOut (int idxFrom, int idxTo) const
{
   const SxArrayList<int,int> &edgesOut = nodes(idxFrom).out;
   if (sortEdges)  {
      ssize_t l = 0;
      ssize_t r = edgesOut.size - 1;
      while (r >= l)  {
         const ssize_t m = (l + r) / 2;
         if (idxTo == edgesOut.elements[m]) return m;
         if (idxTo <  edgesOut.elements[m]) r = m - 1;
         else                               l = m + 1;
      }
   }  else  {
      const ssize_t n = edgesOut.size;
      for (ssize_t i=0; i < n; ++i)  {
         if (idxTo == edgesOut.elements[i]) return i;
      }
   }
   return -1;
}

template<class T>
ssize_t SxGraph<T,SxNull>::findEdgeIn (int idxFrom, int idxTo) const
{
   if (sortEdges)  {
      return nodes(idxFrom).in.binarySearch (idxTo);
   }  else  {
      return  nodes(idxFrom).in.findPos (idxTo);
   }
}

template<class T>
void SxGraph<T,SxNull>::disconnectOut (int idxFrom, int idxTo)
{
   // --- remove all outgoing
   SxArrayList<int,int> &edgesOut = nodes(idxFrom).out;
   if (sortEdges)  {
      ssize_t idx1 = -1;
      ssize_t idx2 = -1;
      ssize_t l = 0;
      ssize_t r = edgesOut.size - 1;
      while (r >= l)  {
         const ssize_t m = (l + r) / 2;
         if (idxTo == edgesOut.elements[m]) {
            idx1 = idx2 = m;
            break;
         }
         if (idxTo < edgesOut.elements[m]) r = m - 1;
         else                              l = m + 1;
      }
      if (idx1 >= 0)  {
         // --- connected directly
         r = edgesOut.size - 1;
         while (idx1 > 0 && edgesOut.elements[idx1 - 1] == idxTo)  --idx1;
         while (idx2 < r && edgesOut.elements[idx2 + 1] == idxTo)  ++idx2;
         ssize_t removeNEdges = (idx2 - idx1) + 1;
         edgesOut.remove (idx1, removeNEdges);
         
         nEdges -= removeNEdges;
      }
   }  else  {
      ssize_t removeNEdges = 0;
      ssize_t idx = -1;
      const ssize_t n = edgesOut.size;
      for (ssize_t i=0; i < n; ++i)  {
         if (idxTo != edgesOut.elements[i])  {
            if (idx >= 0)  {
               // --- shift to empty place from removed edges
               edgesOut.elements[idx] = edgesOut.elements[i];
               idx++;
            }
         }  else  {
            removeNEdges++;
            if (idx < 0) idx = i; // insert place
         }
      }
      if (removeNEdges > 0)  {
         nEdges -= removeNEdges;
         edgesOut.remove (n - removeNEdges, removeNEdges);
      }
   }
}

template<class T>
void SxGraph<T,SxNull>::disconnectIn (int idxFrom, int idxTo)
{
   // --- remove incomming
   SxArrayList<int,int> &edgesIn = nodes(idxTo).in;
   ssize_t idx = -1;
   if (sortEdges)  {
      idx = edgesIn.binarySearch (idxFrom);
   }  else  {
      idx = edgesIn.findPos (idxFrom);
   }
   if (idx >= 0)  {
      // --- connected directly
      edgesIn.remove (idx);
   }
}

template<class T>
void SxGraph<T,SxNull>::removeSelection (const Selection &selection)
{
   if (selection.getPtr ())  {
      SxList<int>::ConstIterator it = selection->begin ();
      while (it.valid ())  {
         // --- invalid selection
         //     (*it) < 0 || (*it) >= nodes.getSize ()
         //     || (*it) is in firstFree list
         removeNode (*it);
         ++it;
      }
   }
}

template<class T>
void SxGraph<T,SxNull>::removeAll ()
{
   nodes.removeAll ();
   
   firstElement = -1;
   lastElement = -1;
   firstFree = -1;
   nNodes = 0;
   nEdges = 0;
   
   hashTable.removeAll ();
   hashSize = 0;
   hashUsed = 0;
   hashBound = 0;
}

template<class T>
bool SxGraph<T,SxNull>::contains (const T &elem) const
{
   return findPos (elem) >= 0;
}

template<class T>
int SxGraph<T,SxNull>::findPath (const T         &from,
                                     const T         &to,
                                     int             maxPathLen,
                                     const Selection &selection) const
{
   int idxFrom = findPos (from);
   if (idxFrom < 0 || (selection.getPtr () && !selection->contains (idxFrom)))
      return 0;
   int idxTo = findPos (to);
   if (idxTo < 0 || (selection.getPtr () && !selection->contains (idxTo)))
      return 0;
   
   if (findEdgeOut (idxFrom, idxTo) >= 0)  {
      return 1;
   }
   
   int pathLen = 0;
   SxSet<int> visited;
   SxList<SxPair<int,int> > queue;
   
   visited << idxFrom;
   nextLevel (idxFrom, pathLen, maxPathLen, selection, &queue, &visited);
   
   while (queue.getSize () > 0)  {
      int ptr = queue.first().key;
      pathLen = queue.first().value;
      queue.removeFirst ();

      ssize_t idx = findEdgeOut (ptr, idxTo);
      if (idx >= 0)  {
         return pathLen + 1;
      }
      nextLevel (ptr, pathLen, maxPathLen, selection, &queue, &visited);
   }

   return 0;
}

template<class T>
SxList<T> SxGraph<T,SxNull>::getPath (const T         &from,
                                          const T         &to,
                                          int             maxPathLen,
                                          const Selection &selection) const
{
   SxList<T> res;

   int idxFrom = findPos (from);
   if (idxFrom < 0 || (selection.getPtr () && !selection->contains (idxFrom)))
      return res;
   int idxTo = findPos (to);
   if (idxTo < 0 || (selection.getPtr () && !selection->contains (idxTo)))
      return res;
   
   if (findEdgeOut (idxFrom, idxTo) >= 0)  {
      res.append (nodes(idxFrom).element);
      res.append (nodes(idxTo).element);
   }  else  {
      int pathLen = 0;
      SxSet<int> visited;
      SxList<SxPair<int,int> > queue;
      
      visited << idxFrom;
      nextLevel (idxFrom, pathLen, maxPathLen, selection, &queue, &visited);
      
      while (queue.getSize () > 0)  {
         int ptr = queue.first().key;
         pathLen = queue.first().value;
         queue.removeFirst ();

         ssize_t idx = findEdgeOut (ptr, idxTo);
         if (idx >= 0)  {
            // --- reverse iteration to get the elements in path
            res.append (nodes(idxTo).element);
            visited.removeElement (idxTo);
            while (ptr != idxFrom)  {
               res.prepend (nodes(ptr).element);
               visited.removeElement (ptr);
               // --- parent
               const SxArrayList<int,int> &edgesIn = nodes(ptr).in;
               ssize_t n = edgesIn.size;
               bool found = false;
               for (ssize_t i=0; i < n; ++i)  {
                  if (visited.contains (edgesIn(i)))  {
                     ptr = edgesIn(i);
                     found = true;
                     break;
                  }
               }
               SX_CHECK (found); // inf loop
            }
            res.prepend (nodes(ptr).element);
            break;
         }
         nextLevel (ptr, pathLen, maxPathLen, selection, &queue, &visited);
      }
   }
   return res;
}

template<class T>
void SxGraph<T,SxNull>::nextLevel (int                      ptr,
                                       int                      pathLen,
                                       int                      maxPathLen,
                                       const Selection          &selection,
                                       SxList<SxPair<int,int> > *queue,
                                       SxSet<int>               *visited) const
{
   if (maxPathLen == 0 || pathLen + 1 < maxPathLen)  {
      SX_CHECK (ptr >= 0);
      SX_CHECK (queue);
      SX_CHECK (visited);

      // --- search in children
      const SxArrayList<int,int> &edgesOut = nodes(ptr).out;
      if (selection.getPtr ())  {
         for (int i=0; i < edgesOut.size; i++)  {
            int key = edgesOut.elements[i];
            if (selection->contains (key) && !visited->contains (key))  {
               queue->append (SxPair<int,int>(key, pathLen + 1));
               visited->append (key);
            }
         }
      }  else  {
         for (int i=0; i < edgesOut.size; i++)  {
            int key = edgesOut.elements[i];
            if (!visited->contains (key))  {
               queue->append (SxPair<int,int>(key, pathLen + 1));
               visited->append (key);
            }
         }
      }
   }
}

template<class T>
SxList<T> SxGraph<T,SxNull>::topsort (const Selection &selection_)
   const
{
   SxList<T> res;
   SxList<int> s;
   int ptr = 0;
   
   // --- number of incoming edges for each vertex and start nodes
   ssize_t len = nodes.mapSize * (nodes.lastBlockIdx + 1);
   SxArray<int> edges(len);
   edges.set (0);
   if (selection_.getPtr ())  {
      typename SxList<int>::ConstIterator it;
      for (it = selection_->begin(); it != selection_->end(); ++it)  {
         int nSelected = 0;
         const SxArrayList<int,int> &edgesIn = nodes(*it).in;
         for (int i=0; i < edgesIn.size; i++)  {
            if (selection_->contains (edgesIn(i)))  {
               nSelected++;
            }
         }
         edges(*it) = nSelected;
         if (nSelected < 1)  s << (*it);
      }
   }  else  {
      ptr = firstElement;
      while (ptr >= 0)  {
         edges(ptr) = nodes(ptr).in.size;
         if (nodes(ptr).in.size < 1)  s << ptr;
         ptr = nodes(ptr).next;
      }
   }

   // --- remove incoming edges and collect new start nodes
   while (s.getSize () > 0)  {
      ptr = s.first ();
      s.removeFirst ();
      res << nodes(ptr).element;
      
      const SxArrayList<int,int> &edgesOut = nodes(ptr).out;
      for (int i=0; i < edgesOut.size; i++)  {
         int idxTo = edgesOut(i);
         if (edges(idxTo) > 0)  {
            edges(idxTo)--;
            if (edges(idxTo) < 1)  s << idxTo;
         }
      }
   }
   
   // --- check cycles
   for (ssize_t i = 0; i < len; ++i)  {
      if (edges(i) > 0)  {
         SX_THROW ("graph has a cycle");
      }
   }            
   return res;
}

// Enumerating Circuits and Loops in Graphs with Self-Arcs and Multiple-Arcs
// K.A. Hawick and H.A. James
// Proc. 2008 Int. Conf. on Foundations of Computer Science (FCS’08), p 14-20
// 2008
template<class T>
SxList<SxList<int> > SxGraph<T,SxNull>::getCyclesIdx (
   const Selection &selection_) const
{
   SxList<SxList<int> > circuits;
   SxArray<bool> visited;
   SxArray<bool> blocked;
   SxArray<SxArrayList<int> > B; // recursive block/unblock
   SxList<int> stack;            // curent circuit
   
   ssize_t len = nodes.mapSize * (nodes.lastBlockIdx + 1);
   blocked.resize (len);
   B.resize (len);
   visited.resize (len);
   
   // --- test circuit in each node
   if (selection_.getPtr ())  {
      if (len > 0)  visited.set (true);
      typename SxList<int>::ConstIterator it;
      for (it = selection_->begin(); it != selection_->end(); ++it)  {
         visited(*it) = false;
      }
      for (it = selection_->begin(); it != selection_->end(); ++it)  {
         blocked.set (false);
         for (ssize_t i=0; i < len; i++)  {
            if (B(i).getSize () > 0) B(i).removeAll ();
         }
         circuit (*it, *it, stack, visited, blocked, B, &circuits);
         visited(*it) = true;
      }
   }  else  {
      if (len > 0)  visited.set (false);
      int ptr = firstElement;
      while (ptr >= 0)  {
         blocked.set (false);
         for (ssize_t i=0; i < len; i++)  {
            if (B(i).getSize () > 0) B(i).removeAll ();
         }
         circuit (ptr, ptr, stack, visited, blocked, B, &circuits);
         visited(ptr) = true;
         ptr = nodes(ptr).next;
      }
   }
   return circuits;
}

template<class T>
SxArray<SxArray<T> > SxGraph<T,SxNull>::getCycles (
   const Selection &selection_) const
{
   SxList<SxList<int> > circuits = getCyclesIdx (selection_);
   
   // --- convert graph indices to elements in found circuits
   SxArray<SxArray<T> > res;
   res.resize (circuits.getSize ());
   SxList<SxList<int> >::ConstIterator it;
   ssize_t i = 0;
   for (it = circuits.begin(); it != circuits.end(); ++it)  {
      res(i).resize (it->getSize ());
      SxList<int>::ConstIterator it2;
      ssize_t j = 0;
      for (it2 = it->begin(); it2 != it->end(); ++it2)  {
         res(i)(j) = nodes(*it2).element;
         j++;
      }
      i++;
   }
   return res;
}

// getCycles
template<class T>
bool SxGraph<T,SxNull>::circuit (int                           idx,
                                     int                           start,
                                     SxList<int>                   &stack,
                                     SxArray<bool>                 &visited,
                                     SxArray<bool>                 &blocked,
                                     SxArray<SxArrayList<int> >    &B,
                                     SxList<SxList<int> >          *circuits)
   const
{
   SX_CHECK (circuits);
   
   bool res = false;
   stack.append (idx);
   blocked(idx) = true;
   
   const SxArrayList<int,int> &edgesOut = nodes(idx).out;
   for (int i=0; i < edgesOut.size; i++)  {
      int to = edgesOut(i);
      if (!visited(to))  {
         if (to == start)  {
            // --- new circuit
            // A -> A, A -> B -> A, A -> B -> C -> A, ...
            circuits->append (stack);
            res = true;
         }  else if (!blocked(to))  {
            if (circuit (to, start, stack, visited, blocked, B, circuits))  {
               res = true;
            }
         }
      }
   }
   
   if (res)  {
      unblock (idx, blocked, B);
   }  else  {
      for (int i=0; i < edgesOut.size; i++)  {
         int to = edgesOut(i);
         if (!visited(to) && !B(to).contains (idx))  {
            B(to).append (idx);
         }
      }
   }
   
   stack.removeLast ();
   return res;
}

// getCycles
template<class T>
void SxGraph<T,SxNull>::unblock (int idx,
                                     SxArray<bool> &blocked,
                                     SxArray<SxArrayList<int> > &B) const
{
   blocked(idx) = false;
   
   for (ssize_t i=0; i < B(idx).size; i++)  {
      int e = B(idx)(i);
      B(idx).remove (e);
      i--;
      if (blocked(e))  {
         unblock (e, blocked, B);
      }
   }
}

/*template<class T>
SxMatrix<Int> SxGraph<T,SxNull>::toMatrix () const
{
   SxMatrix<Int> result;

   ssize_t n = getSize ();
   if (n > 0)  {
      result.reformat (n, n);
      result.set (0);

      // --- create map: node 'pointer' to node order   
      ssize_t len = nodes.mapSize * (nodes.lastBlockIdx + 1);
      SxArray<int> idx;
      idx.resize (len);
      int i = 0;
      int ptr = firstElement;
      while (ptr >= 0)  {
         idx(ptr) = i++;
         ptr = nodes(ptr).next;
      }
      
      // --- fill matrix with direct neighbors
      i = 0;
      ptr = firstElement;
      while (ptr >= 0)  {
         const SxArrayList<int, int> &edgesOut = nodes(ptr).out;
         for (int e=0; e < edgesOut.size; e++)  {
            result(idx(edgesOut(e)), i) = 1;
         }
         i++;
         ptr = nodes(ptr).next;
      }
   }
   
   return result;
}

template<class T>
SxMatrix<Int> SxGraph<T,SxNull>::transitiveClosure () const
{
   SxMatrix<Int> m = toMatrix ();
   
   // --- Floyd–Warshall algorithm
   ssize_t n = m.nRows ();
   for (ssize_t k=0; k < n; ++k)
      for (ssize_t i=0; i < n; ++i)
         for (ssize_t j=0; j < n; ++j)
            if (m(j, k) > 0 && m(k, i) > 0)
               m(j, i) = 1;
   
   return m;
}
*/

template<class T>
void SxGraph<T,SxNull>::print (
   typename SxGraph<T,SxNull>::Iterator it,
   std::ostream                             &s,
   const SxString                           &opt) const
{
   SxUniqueList<int> visited;
   typename SxGraph<T,SxNull>::Iterator itEnd;
   for (; it != itEnd; ++it)  {
      visited << it.getIdx ();
   }
   print (visited, s, opt);
}

template<class T>
void SxGraph<T,SxNull>::print (
   typename SxGraph<T,SxNull>::ConstIterator it,
   std::ostream                                  &s,
   const SxString                                &opt) const
{
   SxUniqueList<int> visited;
   typename SxGraph<T,SxNull>::ConstIterator itEnd;
   for (; it != itEnd; ++it)  {
      visited << it.getIdx ();
   }
   print (visited, s, opt);
}

template<class T>
void SxGraph<T,SxNull>::print (
   const SxUniqueList<int> &visited,
   std::ostream            &s,
   const SxString          &opt) const
{
   s << "digraph G {" << endl;
   if (opt != "")  {
      s << opt << endl;
   }
   
   typename SxList<int>::ConstIterator itIdx;
   for (itIdx = visited.begin(); itIdx != visited.end(); ++itIdx)  {
      const Node *node = &nodes(*itIdx);
      if (node->out.size < 1 && node->in.size < 1)  {
         s << "   \"" << node->element << "\"" << endl;
      }  else  {
         const SxArrayList<int,int> &edgesOut = node->out;
         bool someEdgeVisited = false;
         for (int i=0; i < edgesOut.size; i++)  {
            if (visited.contains (edgesOut(i)))  {
               s << "   \"" << node->element << "\" -> \""
                 << nodes(edgesOut(i)).element << "\"" << endl;
               someEdgeVisited = true;
            }
         }
         if (!someEdgeVisited && edgesOut.getSize () > 0)
            s << "   \"" << node->element << "\"" << endl;
      }
   }
   s << "}" << endl;
}

template<class T>
void SxGraph<T,SxNull>::printDebug () const
{
   size_t nBytesNodes = 0;
   size_t nBytesEdges = 0;
   int ptr = firstElement;
   while (ptr >= 0)  {
      nBytesNodes += sizeof(nodes(ptr))
                   + ::getNBytes (nodes(ptr).element)
                         - sizeof(nodes(ptr).element);
      const SxArrayList<int,int> &edgesOut = nodes(ptr).out;
      for (int i=0; i < edgesOut.size; i++)  {
         nBytesEdges += ::getNBytes (edgesOut.elements[i]);
      }
      nBytesEdges += sizeof(int) * nodes(ptr).in.size;
      ptr = nodes(ptr).next;
   }

                       
   cout << "nodes : " << nNodes << " ... " << nBytesNodes << " bytes" << endl;
   cout << "edges : " << nEdges << " ... " << nBytesEdges << " bytes" << endl;
   cout << "mem   : " << getNBytes() << " bytes" << endl;
   sxprintf ("ptr   : %p\n", (void*)nodes.map);
}

template<class T>
int SxGraph<T,SxNull>::findPos (const T &elem) const
{
   ssize_t idx = hashFindPos (elem);
   return (idx >= 0) ? hashTable.elements[idx] : -1;
}

template<class T>
ssize_t SxGraph<T,SxNull>::hashFindPos (const T &elem) const
{
   if (hashSize > 0)  {
      size_t h = sxHash (elem);
      ssize_t idx = h % hashTable.size;
      ssize_t step = h % (hashTable.size - 1) + 1;
      ssize_t lastIdx = idx;
      
      do {
         if (hashTable.elements[idx] == -1)  {
            return -1;
         }  else if (hashTable.elements[idx] != -2 &&
                     nodes(hashTable.elements[idx]).element == elem)
         {
            return idx;
         }
         idx = (idx + step) % hashTable.size;
      } while (idx != lastIdx);
   }
   return -1;
}

template<class T>
bool SxGraph<T,SxNull>::hashContains (const T &elem_, ssize_t *result_)
{
   SX_CHECK (result_);

   if (hashUsed >= hashBound)  {
      if (hashSize * 2 < hashTable.size)  {
         hashResize (hashTable.size - 1);
      }  else  {
         hashResize (hashTable.size + 1);
      }
   }
   
   size_t h = sxHash (elem_);
   ssize_t idx = h % hashTable.size;
   ssize_t step = h % (hashTable.size - 1) + 1;
   ssize_t lastIdx = idx;
   ssize_t replaceIdx = hashTable.size;
   
   do {
      if (hashTable.elements[idx] == -1)  {
         // --- insert new element
         if (replaceIdx != hashTable.size)  {
            idx = replaceIdx;
         }  else  {
            hashUsed++;
         }
         hashSize++;
         *result_ = idx;
         return false;
      }  else if (hashTable.elements[idx] == -2)  {
         replaceIdx = idx;
      }  else if (nodes(hashTable.elements[idx]).element == elem_)  {
         *result_ = idx;
         return true;
      }
      idx = (idx + step) % hashTable.size;
   } while (idx != lastIdx);
   
   throw SxException ("Can not insert.", __FILE__, __LINE__);
}

template<class T>
void SxGraph<T,SxNull>::hashResize (ssize_t newSize_)
{
   if (newSize_ >= 1610612741)  {
      throw SxException ("HashTable hashSize is too big", __FILE__, __LINE__);
   }
   
   // --- http://planetmath.org/encyclopedia/GoodHashTablePrimes.html
   ssize_t prime[30] = {0, 3, 11, 23, 53, 97, 193, 389, 769, 1543, 3079, 6151,
                        12289, 24593, 49157, 98317, 196613, 393241, 786433,
                        1572869, 3145739, 6291469, 12582917, 25165843, 50331653,
                        100663319, 201326611, 402653189, 805306457, 1610612741};
   ssize_t idx = 0;
   while (prime[idx] <= newSize_)  {
      idx++;
   }
   newSize_ = prime[idx];
   ssize_t newBound = (ssize_t)((double)newSize_ * 0.77 + 0.5);
   ssize_t n = hashTable.size;
   ssize_t i;
   
   if (hashSize < newBound)  {
      SxArray<int> newTable(newSize_);
      if (newSize_ > 0)  {
         newTable.set (-1);
      }
      
      for (i = 0; i < n; i++)  {
         if (hashTable.elements[i] >= 0)  {
            size_t h = sxHash (nodes(hashTable.elements[i]).element);
            idx = h % newSize_;
            ssize_t step = h % (newSize_ - 1) + 1;
            ssize_t lastIdx = idx;
            
            while (newTable.elements[idx] >= 0)  {
               idx = (idx + step) % newSize_;
               if (idx == lastIdx)  {
                  SX_EXIT;
               }
            }
            newTable.elements[idx] = hashTable.elements[i];
         }
      }
            
      hashUsed = hashSize;
      hashBound = newBound;
      hashTable.resize (newSize_, false);
      size_t len = sizeof(int) * (size_t)hashTable.size;
      ::memcpy (hashTable.elements, newTable.elements, len);
   }
}

template<class T>
size_t SxGraph<T,SxNull>::getNBytes () const
{
   size_t nBytes = sizeof(*this)
                 + ::getNBytes (hashTable) - sizeof(hashTable);
   
   for (ssize_t i = 0; i < nodes.mapSize; i++)  {
      // --- size of one segment
      nBytes += sizeof (Node *); // sizeof map[i]
      const Node *block = nodes.map[i];
      SX_CHECK (block);
      for (ssize_t j=0; j <= nodes.lastBlockIdx; j++)  {
         nBytes += sizeof(block[j])
                 + ::getNBytes (block[j].element)
                       - sizeof(block[j].element)
                 + ::getNBytes (block[j].in)
                       - sizeof(block[j].in)
                 + ::getNBytes (block[j].out)
                       - sizeof(block[j].out);
      }
   }
              
   return nBytes;
}

template<class T>
size_t getNBytes (const SxGraph<T,SxNull> &in)
{
   return in.getNBytes ();
}
// ----------------------------------------------------------------------------
template<class T>
SxGraph<T,SxNull>::Iterator::Iterator ()
   : graph(NULL),
     node(NULL),
     idx(-1),
     follow(0),
     pathLen(0),
     selectedSizeIn(-1),
     selectedSizeOut(-1)
{
   // empty
}

template<class T>
SxGraph<T,SxNull>::ConstIterator::ConstIterator ()
   : graph(NULL),
     node(NULL),
     idx(-1),
     follow(0),
     pathLen(0),
     selectedSizeIn(-1),
     selectedSizeOut(-1)
{
   // empty
}

template<class T>
SxGraph<T,SxNull>::Iterator::Iterator (SxGraph<T,SxNull> *graph_,
                                           int                   idx_,
                                           int                   follow_,
                                           const Selection       &selection_)
   : graph(graph_),
     selection(selection_),
     node(NULL),
     idx(idx_),
     follow(follow_),
     pathLen(0),
     selectedSizeIn(-1),
     selectedSizeOut(-1)
{
   SX_CHECK (graph);
   
   if (follow)  {
      if (idx >= 0)  {
         queue << SxPair<int,int>(idx, pathLen);
         visited << idx;
         next ();
      }
   }  else  {
      if (selection.getPtr ())  {
         selectionIt = selection->begin ();
         if (selectionIt.valid ()) ++selectionIt;
      }
      node = (idx >= 0) ? &graph->nodes(idx) : NULL;
   }
}

template<class T>
SxGraph<T,SxNull>::ConstIterator::ConstIterator (
   const SxGraph<T,SxNull> *graph_,
   int                   idx_,
   int                   follow_,
   const Selection       &selection_)
   : graph(graph_),
     selection(selection_),
     node(NULL),
     idx(idx_),
     follow(follow_),
     pathLen(0),
     selectedSizeIn(-1),
     selectedSizeOut(-1)
{
   SX_CHECK (graph);
   
   if (follow)  {
      if (idx >= 0)  {
         queue << SxPair<int,int>(idx, pathLen);
         visited << idx;
         next ();
      }
   }  else  {
      if (selection.getPtr ())  {
         selectionIt = selection->begin ();
         if (selectionIt.valid ()) ++selectionIt;
      }
      node = (idx >= 0) ? &graph->nodes(idx) : NULL;
   }
}

template<class T>
bool SxGraph<T,SxNull>::Iterator::operator!= (
   const typename SxGraph<T,SxNull>::Iterator &it)
{
   return idx != it.idx;
}

template<class T>
bool SxGraph<T,SxNull>::ConstIterator::operator!= (
   const typename SxGraph<T,SxNull>::ConstIterator &it) const
{
   return idx != it.idx;
}

template<class T>
typename SxGraph<T,SxNull>::Iterator &
SxGraph<T,SxNull>::Iterator::operator++ ()
{
   SX_CHECK (graph);
   if (follow)  {
      next ();
   }  else if (selection.getPtr ()) {
      if (selectionIt.valid ())  {
         idx = *selectionIt;
         node = &graph->nodes(idx);
         ++selectionIt;
      }  else  {
         idx = -1;
         node = NULL;
      }
   }  else  {
      idx = node->next;
      node = (idx >= 0) ? &graph->nodes(idx) : NULL;
   }
   // --- reset neighbors in selection
   selectedSizeIn = -1;
   selectedSizeOut = -1;
   return *this;
}

template<class T>
typename SxGraph<T,SxNull>::ConstIterator &
SxGraph<T,SxNull>::ConstIterator::operator++ ()
{
   SX_CHECK (graph);
   if (follow)  {
      next ();
   }  else if (selection.getPtr ()) {
      if (selectionIt.valid ())  {
         idx = *selectionIt;
         node = &graph->nodes(idx);
         ++selectionIt;
      }  else  {
         idx = -1;
         node = NULL;
      }
   }  else  {
      idx = node->next;
      node = (idx >= 0) ? &graph->nodes(idx) : NULL;
   }
   // --- reset neighbors in selection
   selectedSizeIn = -1;
   selectedSizeOut = -1;
   return *this;
}

template<class T>
void SxGraph<T,SxNull>::Iterator::next ()
{
   if (queue.getSize () < 1)  {
      idx  = -1;
      node = NULL;
      return;
   }
   
   idx     = queue.first().key;
   if (idx < 0)  {
      node = NULL;
      return;
   }
   node    = &graph->nodes(idx);
   pathLen = queue.first().value;
   queue.removeFirst ();
   
   if (follow & 1)  {
      // --- follow outgouing
      const SxArrayList<int,int> &edges = node->out;
      ssize_t n = edges.size;
      if (selection.getPtr ())  {
         for (ssize_t i=0; i < n; ++i)  {
            if (  !visited.contains (edges(i))
                && selection->contains (edges(i)))
            {
               queue << SxPair<int,int>(edges(i), pathLen + 1);
               visited << edges(i);
            }
         }
      }  else  {
         for (ssize_t i=0; i < n; ++i)  {
            if (!visited.contains (edges(i)))  {
               queue << SxPair<int,int>(edges(i), pathLen + 1);
               visited << edges(i);
            }
         }
      }
   }
   if (follow & 2)  {
      // --- follow incoming
      const SxArrayList<int,int> &edges = node->in;
      ssize_t n = edges.size;
      if (selection.getPtr ())  {
         for (ssize_t i=0; i < n; ++i)  {
            if (  !visited.contains (edges(i))
                && selection->contains (edges(i)))
            {
               queue << SxPair<int,int>(edges(i), pathLen + 1);
               visited << edges(i);
            }
         }
      }  else  {
         for (ssize_t i=0; i < n; ++i)  {
            if (!visited.contains (edges(i)))  {
               queue << SxPair<int,int>(edges(i), pathLen + 1);
               visited << edges(i);
            }
         }
      }
   }
}

template<class T>
void SxGraph<T,SxNull>::ConstIterator::next ()
{
   if (queue.getSize () < 1)  {
      idx  = -1;
      node = NULL;
      return;
   }
   
   idx     = queue.first().key;
   if (idx < 0)  {
      node = NULL;
      return;
   }
   node    = &graph->nodes(idx);
   pathLen = queue.first().value;
   queue.removeFirst ();
   
   if (follow & 1)  {
      // --- follow outgouing
      const SxArrayList<int,int> &edges = node->out;
      ssize_t n = edges.size;
      if (selection.getPtr ())  {
         for (ssize_t i=0; i < n; ++i)  {
            if (  !visited.contains (edges(i))
                && selection->contains (edges(i)))
            {
               queue << SxPair<int,int>(edges(i), pathLen + 1);
               visited << edges(i);
            }
         }
      }  else  {
         for (ssize_t i=0; i < n; ++i)  {
            if (!visited.contains (edges(i)))  {
               queue << SxPair<int,int>(edges(i), pathLen + 1);
               visited << edges(i);
            }
         }
      }
   }
   if (follow & 2)  {
      // --- follow incoming
      const SxArrayList<int,int> &edges = node->in;
      ssize_t n = edges.size;
      if (selection.getPtr ())  {
         for (ssize_t i=0; i < n; ++i)  {
            if (  !visited.contains (edges(i))
                && selection->contains (edges(i)))
            {
               queue << SxPair<int,int>(edges(i), pathLen + 1);
               visited << edges(i);
            }
         }
      }  else  {
         for (ssize_t i=0; i < n; ++i)  {
            if (!visited.contains (edges(i)))  {
               queue << SxPair<int,int>(edges(i), pathLen + 1);
               visited << edges(i);
            }
         }
      }
   }
}
//template<class T>
//bool SxGraph<T,SxNull>::Iterator::setParent ()
//{
//   SX_CHECK (node);
//   const SxArrayList<int,int> &edges = node->in;
//   ssize_t n = edges.size;
//   for (ssize_t i=0; i < n; ++i)  {
//      if (visited.contains (edges(i)))  {
//         idx  = edges(i);
//         node = &graph->nodes(idx);
//         // --- reset neighbors in selection
//         selectedSizeIn = -1;
//         selectedSizeOut = -1;
//         return true;
//      }
//   }
//   return false;
//}

template<class T>
typename SxGraph<T,SxNull>::Selection
SxGraph<T,SxNull>::Iterator::getUnvisited () const
{
   SX_CHECK (graph);
   Selection result;

   typename SxGraph<T>::Iterator it = graph->begin ();
   while (it.valid ())  {
      int nodeId = it.idx;
      if (!visited.contains (nodeId))  {
         if (!result.getPtr ())
            result = Selection::create ();
         result->append (nodeId);
      }
      ++it;
   }
   return result;
}

template<class T>
typename SxGraph<T,SxNull>::Selection
SxGraph<T,SxNull>::ConstIterator::getUnvisited () const
{
   SX_CHECK (graph);
   Selection result;

   typename SxGraph<T>::ConstIterator it = graph->begin ();
   while (it.valid ())  {
      int nodeId = it.idx;
      if (!visited.contains (nodeId))  {
         if (!result.getPtr ())
            result = Selection::create ();
         result->append (nodeId);
      }
      ++it;
   }
   return result;
}

template<class T>
T &SxGraph<T,SxNull>::Iterator::operator* ()
{
   SX_CHECK (node);
   return node->element; 
}

template<class T>
T *SxGraph<T,SxNull>::Iterator::operator-> ()
{
   SX_CHECK (node);
   return &node->element; 
}

template<class T>
const T &SxGraph<T,SxNull>::ConstIterator::operator* () const
{
   SX_CHECK (node);
   return node->element; 
}

template<class T>
const T *SxGraph<T,SxNull>::ConstIterator::operator-> () const
{
   SX_CHECK (node);
   return &node->element; 
}

template<class T>
T &SxGraph<T,SxNull>::Iterator::in (ssize_t idx_)
{
   SX_CHECK (node);
   if (selection.getPtr ())  {
      if (selectedSizeIn < 0)  {
         // --- build cache of selected neighbors
         selectedSizeIn = 0;
         const SxArrayList<int,int> &edgesIn = node->in;
         ssize_t n = edgesIn.size;
         for (ssize_t i=0; i < n; ++i)  {
            if (selection->contains (edgesIn(i)))  {
               selectedSizeIn++;
            }
         }
         selectedIn.resize (selectedSizeIn);
         selectedSizeIn = 0;
         for (ssize_t i=0; i < n; ++i)  {
            if (selection->contains (edgesIn(i)))  {
               selectedIn(selectedSizeIn) = i;
               selectedSizeIn++;
            }
         }
      }
      return graph->nodes(node->out(selectedIn(idx_))).element;
   }
   return graph->nodes(node->in(idx_)).element;
}

template<class T>
const T &SxGraph<T,SxNull>::ConstIterator::in (ssize_t idx_) const
{
   SX_CHECK (node);
   if (selection.getPtr ())  {
      if (selectedSizeIn < 0)  {
         // --- build cache of selected neighbors
         selectedSizeIn = 0;
         const SxArrayList<int,int> &edgesIn = node->in;
         ssize_t n = edgesIn.size;
         for (ssize_t i=0; i < n; ++i)  {
            if (selection->contains (edgesIn(i)))  {
               selectedSizeIn++;
            }
         }
         selectedIn.resize (selectedSizeIn);
         selectedSizeIn = 0;
         for (ssize_t i=0; i < n; ++i)  {
            if (selection->contains (edgesIn(i)))  {
               selectedIn(selectedSizeIn) = i;
               selectedSizeIn++;
            }
         }
      }
      return graph->nodes(node->out(selectedIn(idx_))).element;
   }
   return graph->nodes(node->in(idx_)).element;
}

template<class T>
T &SxGraph<T,SxNull>::Iterator::out (ssize_t idx_)
{
   SX_CHECK (node);
   if (selection.getPtr ())  {
      if (selectedSizeOut < 0)  {
         // --- build cache of selected neighbors
         selectedSizeOut = 0;
         const SxArrayList<int,int> &edgesOut = node->out;
         ssize_t n = edgesOut.size;
         for (ssize_t i=0; i < n; ++i)  {
            if (selection->contains (edgesOut(i)))  {
               selectedSizeOut++;
            }
         }
         selectedOut.resize (selectedSizeOut);
         selectedSizeOut = 0;
         for (ssize_t i=0; i < n; ++i)  {
            if (selection->contains (edgesOut(i)))  {
               selectedOut(selectedSizeOut) = i;
               selectedSizeOut++;
            }
         }
      }
      return graph->nodes(node->out(selectedOut(idx_))).element;
   }
   return graph->nodes(node->out(idx_)).element;
}

template<class T>
const T &SxGraph<T,SxNull>::ConstIterator::out (ssize_t idx_) const
{
   SX_CHECK (node);
   if (selection.getPtr ())  {
      if (selectedSizeOut < 0)  {
         // --- build cache of selected neighbors
         selectedSizeOut = 0;
         const SxArrayList<int,int> &edgesOut = node->out;
         ssize_t n = edgesOut.size;
         for (ssize_t i=0; i < n; ++i)  {
            if (selection->contains (edgesOut(i)))  {
               selectedSizeOut++;
            }
         }
         selectedOut.resize (selectedSizeOut);
         selectedSizeOut = 0;
         for (ssize_t i=0; i < n; ++i)  {
            if (selection->contains (edgesOut(i)))  {
               selectedOut(selectedSizeOut) = i;
               selectedSizeOut++;
            }
         }
      }
      return graph->nodes(node->out(selectedOut(idx_))).element;
   }
   return graph->nodes(node->out(idx_)).element;
}

template<class T>
ssize_t SxGraph<T,SxNull>::Iterator::getSizeIn () const
{
   SX_CHECK (node);
   if (selection.getPtr ())  {
      // --- count selected neighbors
      ssize_t nSelected = 0;
      const SxArrayList<int,int> &edgesIn = node->in;
      ssize_t n = edgesIn.size;
      for (ssize_t i=0; i < n; ++i)  {
         if (selection->contains (edgesIn(i)))  {
            nSelected++;
         }
      }
      return nSelected;
   }
   return node->in.size;
}

template<class T>
ssize_t SxGraph<T,SxNull>::ConstIterator::getSizeIn () const
{
   SX_CHECK (node);
   if (selection.getPtr ())  {
      // --- count selected neighbors
      ssize_t nSelected = 0;
      const SxArrayList<int,int> &edgesIn = node->in;
      ssize_t n = edgesIn.size;
      for (ssize_t i=0; i < n; ++i)  {
         if (selection->contains (edgesIn(i)))  {
            nSelected++;
         }
      }
      return nSelected;
   }
   return node->in.size;
}

template<class T>
ssize_t SxGraph<T,SxNull>::Iterator::getSizeOut () const
{
   SX_CHECK (node);
   if (selection.getPtr ())  {
      // --- count selected neighbors
      ssize_t nSelected = 0;
      const SxArrayList<int,int> &edgesOut = node->out;
      ssize_t n = edgesOut.size;
      for (ssize_t i=0; i < n; ++i)  {
         if (selection->contains (edgesOut(i)))  {
            nSelected++;
         }
      }
      return nSelected;
   }
   return node->out.size;
}

template<class T>
ssize_t SxGraph<T,SxNull>::ConstIterator::getSizeOut () const
{
   SX_CHECK (node);
   if (selection.getPtr ())  {
      // --- count selected neighbors
      ssize_t nSelected = 0;
      const SxArrayList<int,int> &edgesOut = node->out;
      ssize_t n = edgesOut.size;
      for (ssize_t i=0; i < n; ++i)  {
         if (selection->contains (edgesOut(i)))  {
            nSelected++;
         }
      }
      return nSelected;
   }
   return node->out.size;
}

template<class T>
ssize_t SxGraph<T,SxNull>::Iterator::findEdgeOut (int idxTo) const
{
   SX_CHECK (idx >= 0);
   return graph->findEdgeOut (idx, idxTo);
}

template<class T>
ssize_t SxGraph<T,SxNull>::Iterator::findEdgeIn (int idxTo) const
{
   SX_CHECK (idx >= 0);
   return graph->findEdgeIn (idx, idxTo);
}

template<class T>
typename SxGraph<T,SxNull>::Iterator
SxGraph<T,SxNull>::begin (const Selection &selection_)
{
   int idx = -1;
   if (selection_.getPtr ())  {
      if (selection_->getSize () > 0) idx = selection_->first ();
   }  else  {
      idx = firstElement;
   }
   return typename SxGraph<T,SxNull>::Iterator (this, idx, 0, selection_);
}

template<class T>
typename SxGraph<T,SxNull>::ConstIterator
SxGraph<T,SxNull>::begin (const Selection &selection_) const
{
   int idx = -1;
   if (selection_.getPtr ())  {
      if (selection_->getSize () > 0) idx = selection_->first ();
   }  else  {
      idx = firstElement;
   }
   return typename SxGraph<T,SxNull>::ConstIterator (this, idx, 0, selection_);
}

template<class T>
typename SxGraph<T,SxNull>::Iterator
SxGraph<T,SxNull>::begin (const T           &elem_,
                              int               follow_,
                              const Selection   &selection_)
{
   int idx = findPos (elem_);
   if (selection_.getPtr () && !selection_->contains (idx)) idx = -1;
   return typename SxGraph<T,SxNull>::Iterator(this,idx,follow_,selection_);
}

template<class T>
typename SxGraph<T,SxNull>::Iterator
SxGraph<T,SxNull>::begin (const T           &elem_,
                              const Selection   &selection_)
{
   int idx = findPos (elem_);
   if (selection_.getPtr () && !selection_->contains (idx)) idx = -1;
   return typename SxGraph<T,SxNull>::Iterator(this,idx,1,selection_);
}

template<class T>
typename SxGraph<T,SxNull>::Iterator
SxGraph<T,SxNull>::beginIn (const T           &elem_,
                                const Selection   &selection_)
{
   int idx = findPos (elem_);
   if (selection_.getPtr () && !selection_->contains (idx)) idx = -1;
   return typename SxGraph<T,SxNull>::Iterator(this,idx,2,selection_);
}

template<class T>
typename SxGraph<T,SxNull>::Iterator
SxGraph<T,SxNull>::beginBoth (const T           &elem_,
                                  const Selection   &selection_)
{
   int idx = findPos (elem_);
   if (selection_.getPtr () && !selection_->contains (idx)) idx = -1;
   return typename SxGraph<T,SxNull>::Iterator(this,idx,3,selection_);
}

template<class T>
typename SxGraph<T,SxNull>::ConstIterator
SxGraph<T,SxNull>::begin (const T           &elem_,
                              const Selection   &selection_) const
{
   int idx = findPos (elem_);
   if (selection_.getPtr () && !selection_->contains (idx)) idx = -1;
   return typename SxGraph<T,SxNull>::ConstIterator(this,idx,1,selection_);
}

template<class T>
typename SxGraph<T,SxNull>::ConstIterator
SxGraph<T,SxNull>::beginIn  (const T           &elem_,
                                 const Selection   &selection_) const
{
   int idx = findPos (elem_);
   if (selection_.getPtr () && !selection_->contains (idx)) idx = -1;
   return typename SxGraph<T,SxNull>::ConstIterator(this,idx,2,selection_);
}

template<class T>
typename SxGraph<T,SxNull>::ConstIterator
SxGraph<T,SxNull>::beginBoth  (const T           &elem_,
                                   const Selection   &selection_) const
{
   int idx = findPos (elem_);
   if (selection_.getPtr () && !selection_->contains (idx)) idx = -1;
   return typename SxGraph<T,SxNull>::ConstIterator(this,idx,3,selection_);
}

template<class T>
typename SxGraph<T,SxNull>::Iterator
SxGraph<T,SxNull>::end ()
{
   return typename SxGraph<T,SxNull>::Iterator ();
}

template<class T>
typename SxGraph<T,SxNull>::ConstIterator
SxGraph<T,SxNull>::end () const
{
   return typename SxGraph<T,SxNull>::ConstIterator ();
}

// ----------------------------------------------------------------------------
template<class T>
SxGraph<T,SxNull>::EdgeIterator::EdgeIterator ()
   : graph(NULL),
     node(NULL),
     edgeIdx(-1),
     singleNode(false)
{
   // empty
}

template<class T>
SxGraph<T,SxNull>::EdgeIterator::EdgeIterator (
   SxGraph<T,SxNull> *graph_,
   int                   idx_,
   const Selection       &selection_)
   : graph(graph_),
     selection(selection_),
     node(NULL),
     edgeIdx(0),
     singleNode(false)
{
   SX_CHECK (graph);
   
   if (idx_ >= 0)  {
      node = &graph->nodes(idx_);
      singleNode = true;
      if (node->out.size < 1)  {
         node = NULL;
      }  else if (    selection.getPtr ()
                  && !selection->contains (node->out(edgeIdx)))
      {
         findNextEdge ();
      }
   }  else  {
      if (selection.getPtr ())  {
         selectionIt = selection->begin (); 
         findNextEdge ();
      }  else  {
         if (graph->firstElement >= 0)  {
            node = &graph->nodes(graph->firstElement);
         }
         // --- find first node with some edges
         while (node && node->out.size < 1)  {
            node = (node->next >= 0) ? &graph->nodes(node->next) : NULL;
         }
      }
   }
}

template<class T>
bool SxGraph<T,SxNull>::EdgeIterator::operator!= (
   const typename SxGraph<T,SxNull>::EdgeIterator &in)
{
   return node != in.node;
}

template<class T>
typename SxGraph<T,SxNull>::EdgeIterator &
SxGraph<T,SxNull>::EdgeIterator::operator++ ()
{
   SX_CHECK (node);

   if (selection.getPtr ())  {
      findNextEdge ();
   }  else  {
      edgeIdx++;
      if (edgeIdx >= node->out.size)  {
         if (singleNode)  {
            node = NULL;
         }  else  {
            // --- find next node with some edges
            edgeIdx = 0;
            do {
               node = (node->next >= 0) ? &graph->nodes(node->next) : NULL;
            }  while (node && node->out.size < 1);
         }
      }
   }
   return *this;
}

template<class T>
void SxGraph<T,SxNull>::EdgeIterator::findNextEdge ()
{
   do {
      if (node && edgeIdx < static_cast<ssize_t>(node->out.size - 1))  {
         edgeIdx++;
      }  else  {
         edgeIdx = 0;
         node = NULL;
         while (selectionIt.valid ())  {
            node = &graph->nodes(*selectionIt);
            ++selectionIt;
            if (node->out.size > 0) break;
            else                    node = NULL;
         }
      }
   }  while (node && !selection->contains (node->out(edgeIdx)));
}

template<class T>
T &SxGraph<T,SxNull>::EdgeIterator::from ()
{
   SX_CHECK (node);
   return node->element;
}

template<class T>
T &SxGraph<T,SxNull>::EdgeIterator::to ()
{
   SX_CHECK (node);
   return graph->nodes(node->out(edgeIdx)).element;
}

template<class T>
typename SxGraph<T,SxNull>::EdgeIterator
SxGraph<T,SxNull>::beginEdge (const Selection &selection_)
{
   return typename SxGraph<T,SxNull>::EdgeIterator (this, -1, selection_);
}

//template<class T>
//typename SxGraph<T,SxNull>::EdgeIterator
//SxGraph<T,SxNull>::beginEdge (const T &elem_,
//                                  SxUniqueList<int> *selection_)
//{
//   int idx = findPos (elem_);
//   if (idx < 0 || (selection_ && !selection_->contains (idx)))
//      return typename SxGraph<T,SxNull>::EdgeIterator ();
//   else
//     return typename SxGraph<T,SxNull>::EdgeIterator(this,idx,selection_);
//}

template<class T>
typename SxGraph<T,SxNull>::EdgeIterator
SxGraph<T,SxNull>::endEdge ()
{
   return typename SxGraph<T,SxNull>::EdgeIterator ();
}

template <class T, class E>
inline std::ostream& operator<< (std::ostream &s, const SxGraph<T,E> &in)
{
   s << "operator<< (SxGraph) not yet implemented" << endl;
   return s;
}

