// ---------------------------------------------------------------------------
//
//      The ab-initio based multiscale library
//
//                  S / P H I / n X
//
//      Copyright:  Max-Planck-Institute for Iron Research
//                  40237 Duesseldorf, Germany
//
//      Contact:    https://sxlib.mpie.de
//      Authors:    see sphinx/AUTHORS
//      License:    see sphinx/LICENSE
//
// ---------------------------------------------------------------------------

#include <SxMPI.h>
#include <SxMPIMgr.h>
#include <SxError.h>

SxMPI::SxMPI (Destination d)
   : SxProcUnit (),
     destination (d),
     rank(0)
{
   init ();
}


SxMPI::SxMPI (const SxPtr<SxMPIMgr> &group_, Destination d)
   : SxProcUnit (group_),
     destination (d),
     rank(0)
{
   init ();
}

SxMPI::~SxMPI ()
{
   // empty
}


void SxMPI::init ()
{
   rank = SxPtr<SxMPIMgr>(group)->updateProcs();
}


void SxMPI::start ()
{
// bool groupMaster = isGroupMaster();
// if (    destination == Anywhere 
//     || (destination == GroupMaster &&  groupMaster)
//     || (destination == GroupSlave  && !groupMaster))
// {
      main ();
// }
}


void SxMPI::wait ()
{
   // empty
}


void SxMPI::barrier ()
{
#  ifdef USE_MPI
      MPI_Barrier (getMPIMgr()->mpiComm);
#  endif
}



bool SxMPI::isMaster () const
{
#  ifdef USE_MPI
      int r;
      MPI_Comm_rank (MPI_COMM_WORLD, &r);
      return (r == 0 && (SxMPIMgr::getGlobalPtr() == getMPIMgr()));
#  else
      return true;
#  endif
}

bool SxMPI::isGroupMaster () const
{
#  ifdef USE_MPI
      int r;
      MPI_Comm_rank (getMPIMgr()->mpiComm, &r);
      return rank == 0;
#  else
      return true;
#  endif
}


SxPtr<SxMPIMgr> SxMPI::getMPIMgr () const
{
   return SxPtr<SxMPIMgr>(group);
}

void SxMPI::main ()
{
   SX_EXIT;  // this function is not pure abstract to allow static linkage
}
