// ---------------------------------------------------------------------------
//
//      The ab-initio based multiscale library
//
//                  S / P H I / n X
//
//      Copyright:  Max-Planck-Institute for Iron Research
//                  40237 Duesseldorf, Germany
//
//      Contact:    https://sxlib.mpie.de
//      Authors:    see sphinx/AUTHORS
//      License:    see sphinx/LICENSE
//
// ---------------------------------------------------------------------------

#ifndef _SX_MPI_MGR_H_
#define _SX_MPI_MGR_H_

#include <SxIPC.h>
#include <SxProcUnitGroup.h>
#include <SxArray.h>
#include <SxPtr.h>

#include <SxConfig.h>
#ifdef USE_MPI
#  include <mpi.h>
#endif

#ifdef USE_MPI
#  define SX_MPI_MASTER_ONLY  if ( SxMPIMgr::getCurProc()->isMaster ())
#  define SX_MPI_SLAVE_ONLY   if (!SxMPIMgr::getCurProc()->isMaster ())
#  define SX_MPI_SHARE(var)   var = SxMPIMgr::getCurProc()->broadcast (var);
#else
#  define SX_MPI_MASTER_ONLY
#  define SX_MPI_SLAVE_ONLY   if (false)
#  define SX_MPI_SHARE(var)
#endif /* USE_MPI */

class SxMPI;
class SxTopology;

class SxMPIHandle
{
   public:
      int rank, nProcs;

      SxMPIHandle (int = 0, char ** = NULL);
     ~SxMPIHandle ();
};


/** \brief ...

    \b SxClass = S/PHI/nX ...

    ....

    \author Sixten Boeck, boeck@mpie.de */
class SX_EXPORT_IPC SxMPIMgr : public SxProcUnitGroup
{
   public:
      SxMPIMgr ();
      SxMPIMgr (const SxPtr<SxMPIMgr> &parent_);
      virtual ~SxMPIMgr ();

      static SxPtr<SxMPIMgr> &getGlobalPtr (int argc=0, char **argv=NULL);

      static int getRank ();
      static int getNProcs ();
      SxArray<SxPtr<SxMPI> > getProcs (bool recursive=false);

      virtual void start ();
      virtual void barrier ();

      virtual SxPtr<SxProcUnitGroup> addGroup (int maxItems=0);

#     ifdef USE_MPI
         static void errorHandlerCB (MPI_Comm *, int *err, ...);
#     endif

      static SxPtr<SxMPI> getCurProc ();

    protected:

      friend class SxMPI;

      SxPtr<SxProcUnitGroup> getTopLevelGroup ();
      SxPtr<SxTopology>      topology;
      SxPtr<SxMPI>           curProc;
      int rank;
      int nProcs;
      bool hasComm;

#     ifdef USE_MPI
         MPI_Errhandler       errHandler;
         MPI_Comm             mpiComm;
         SxPtr<SxMPIHandle>   mpiHandle;
#     endif

      void init ();
      int updateProcs ();
      virtual void deregisterProcUnit (const SxPtr<SxProcUnit> &);
};

#endif /* _SX_MPI_MGR_H_ */
