// ---------------------------------------------------------------------------
//
//      The ab-initio based multiscale library
//
//                  S / P H I / n X
//
//      Copyright:  Max-Planck-Institute for Iron Research
//                  40237 Duesseldorf, Germany
//
//      Contact:    https://sxlib.mpie.de
//      Authors:    see sphinx/AUTHORS
//      License:    see sphinx/LICENSE
//
// ---------------------------------------------------------------------------

#ifndef _SX_PROC_UNIT_H_
#define _SX_PROC_UNIT_H_

#include <SxIPC.h>
#include <SxPtr.h>
#include <SxSignals.h>

class SxProcUnitGroup;

/** \brief ...

    \b SxProcUnit = S/PHI/nX Process Unit

    ....

    \author Sixten Boeck, boeck@mpie.de */
class SX_EXPORT_IPC SxProcUnit : public SxThis<SxProcUnit>
{
   public:

      SxProcUnit ();
      SxProcUnit (const SxPtr<SxProcUnitGroup> &);
      virtual ~SxProcUnit ();

      virtual void start ()=0;
      virtual void wait ()=0;

      //void setId (ssize_t id);
      //ssize_t getId () const;

   signals:

      SxSignal<SxProcUnit *> SX_SIGNAL (sigStarted);
      SxSignal<SxProcUnit *> SX_SIGNAL (sigFinished);

   protected:

     friend class SxTask;
     friend class SxProcUnitGroup;

     ssize_t                 unitId;
     SxPtr<SxProcUnitGroup>  group;

     void reparent (const SxPtr<SxProcUnitGroup> &);

};

#endif /* _SX_PROC_UNIT_H_ */
