// ---------------------------------------------------------------------------
//
//      The ab-initio based multiscale library
//
//                  S / P H I / n X
//
//      Copyright:  Max-Planck-Institute for Iron Research
//                  40237 Duesseldorf, Germany
//
//      Contact:    https://sxlib.mpie.de
//      Authors:    see sphinx/AUTHORS
//      License:    see sphinx/LICENSE
//
// ---------------------------------------------------------------------------

#include <SxTask.h>
#include <SxProcUnit.h>

SxTask::SxTask ()
  : priority(99),
    maxRunTime(-1),
    maxSpoolTime(-1),
    timer (2, true),
    prioPoints (0)
{
   // empty
}

SxTask::SxTask (const SxPtr<SxProcUnit> &task_,
                int priority_,
                double maxRunTime_,
                double maxSpoolTime_)
  : priority(priority_), 
    maxRunTime(maxRunTime_), 
    maxSpoolTime(maxSpoolTime_),
    timer (2, true),
    task(task_),
    prioPoints (0)
{
   SX_CHECK_VAR (priority >= 0 && priority < MaxPriority, priority);
   SX_CHECK (task->group.getPtr() == NULL);
   timer.start (Spooling);
}


SxTask::~SxTask ()
{
   // empty
}


void SxTask::start ()
{
   timer.start (Running);
   task->start ();
}


void SxTask::wait ()
{
   task->wait ();
}


int SxTask::updatePriority ()
{
   ++prioPoints;
   return prioPoints - priority;
}

bool SxTask::spoolingExpired () const
{
   if (maxSpoolTime < 0)  return false;
   return timer.getTime(Spooling) > maxSpoolTime;
}

bool SxTask::runningExpired () const
{
   if (maxRunTime < 0)  return false;
   return timer.getTime(Running) > maxRunTime;
}
