// ---------------------------------------------------------------------------
//
//      The ab-initio based multiscale library
//
//                  S / P H I / n X
//
//      Copyright:  Max-Planck-Institute for Iron Research
//                  40237 Duesseldorf, Germany
//
//      Contact:    https://sxlib.mpie.de
//      Authors:    see sphinx/AUTHORS
//      License:    see sphinx/LICENSE
//
// ---------------------------------------------------------------------------

#ifndef _SX_THREAD_POOL_H_
#define _SX_THREAD_POOL_H_

#include <SxArray.h>
#include <SxPtr.h>
#include <SxThreadCond.h>
#include <SxTask.h>
#include <SxSystemThread.h>
#include <SxIPC.h>

/** \brief Thread Pool

    \b SxThreadPool = S/PHI/nX Thread Pool

    ....

    \author Sixten Boeck, boeck@gemmantics.com
    \author Vaclav Bubnik, bubnik@gemmantics.com */
class SX_EXPORT_IPC SxThreadPool
{
   public:
      SxThreadPool ();
      virtual ~SxThreadPool ();
      
      static void initialize (int nThreads=0); // default all available
      static SxThreadPool &getGlobalObj ();
      
      /** \return the number of resources (System threads) */
      ssize_t getSize () const;
     
      // --- task manager
      void submit (const SxTask &); // #exception
      void submit (const SxPtr<SxProcUnit> &); // #exception
      void operator<< (const SxTask &); // #exception
      bool contains (const SxProcUnit *procUnit) const;
      bool isRunning (const SxProcUnit *procUnit) const;
      void wait (const SxProcUnit *procUnit);
      ssize_t getNTasks () const;
      
      static int getNProcs ();

   protected:
      bool containsWaiting (const SxProcUnit *procUnit) const;
      bool containsRunning (const SxProcUnit *procUnit) const;
      void resizePool (ssize_t nThreads);
      void shutdown ();
      
      class PoolThread : public SxSystemThread
      {
         public:
            SxThreadPool  *pool;
            int            id;
     
            PoolThread (SxThreadPool *pool, int id);
            virtual ~PoolThread ();
            virtual void main ();
      };
     
      SxArray<SxPtr<PoolThread> >  pool;
      SxList<SxTask>               tasks;
      SxList<SxList<SxTask>::List*> waitingTasks;
      SxList<SxList<SxTask>::List*> runningTasks;
      mutable SxMutex              mutex;
      mutable SxThreadCond         condition;
      mutable SxThreadCond         waitCondition;
      bool                         terminateCmd;
};

#endif /* _SX_THREAD_POOL_H_ */
