// ---------------------------------------------------------------------------
//
//      The ab-initio based multiscale library
//
//                  S / P H I / n X
//
//      Copyright:  Max-Planck-Institute for Iron Research
//                  40237 Duesseldorf, Germany
//
//      Contact:    https://sxlib.mpie.de
//      Authors:    see sphinx/AUTHORS
//      License:    see sphinx/LICENSE
//
// ---------------------------------------------------------------------------

#include <SxTopology.h>
#include <SxMPIMgr.h>

SxTopology::SxTopology ()
   : type (SxTopology::Undefined)
{
   SxPtr<SxMPIMgr> mgr = SxMPIMgr::getGlobalPtr ();
   iProc  = mgr->getRank ();
   nProcs = mgr->getNProcs ();
}

SxTopology::SxTopology (int iProc_, int nProcs_, Type type_) 
   : iProc(iProc_),
     nProcs(nProcs_),
     type (type_)
{
   update (nProcs);
}



SxTopology::~SxTopology ()
{
   // empty
}


SxPtr<SxTopology> SxTopology::clone () const
{
   SxPtr<SxTopology> res = SxPtr<SxTopology>::create (iProc, nProcs, type);
   return res;
}


void SxTopology::update (int nProcs_)
{
   if (nProcs == nProcs_)  return;

   nProcs = nProcs_;
   procMap.resize (nProcs);

   if (type == Balanced)  {
      int iProc = 0;
      for (int i=0; i < nProcs; ++i)  {
         procMap(i) = iProc++;
         if (iProc >= nProcs)  iProc = 0;
      }
   }  else if (type == FarmerWorker)  {

      int iProc = 0;
      procMap(0) = iProc++;
      for (int i=1; i < nProcs; ++i)  {
         procMap(i) = iProc++;
         if (iProc >= nProcs)  iProc = (nProcs > 1) ? 1 : 0;
      }
   }  else  {
      SX_EXIT;
   }
}


bool SxTopology::isLocalTask (int iTask) const
{
   return (procMap(iTask) == iProc);
}


int SxTopology::getProc (int iTask) const
{
   SX_CHECK (type != Undefined); // derive from SxTopology to define own map!
   return procMap (iTask);
}
