// ---------------------------------------------------------------------------
//
//      The ab-initio based multiscale library
//
//                  S / P H I / n X
//
//      Copyright:  Max-Planck-Institute for Iron Research
//                  40237 Duesseldorf, Germany
//
//      Contact:    https://sxlib.mpie.de
//      Authors:    see sphinx/AUTHORS
//      License:    see sphinx/LICENSE
//
// ---------------------------------------------------------------------------

#ifndef _SX_TOPOLOGY_H_
#define _SX_TOPOLOGY_H_

#include <SxArray.h>
#include <SxPtr.h>

/** \brief ...

    \b SxTopology = S/PHI/nX Process Topology

    ....

    \author Sixten Boeck, boeck@mpie.de */
class SxTopology
{
   public:

      // predefined topologies. derive from SxTopology to define own types
      enum Type { Undefined, Balanced, FarmerWorker };

      SxTopology ();
      SxTopology (int iProc_, int nProcs_=1, Type type_=Undefined);
      virtual ~SxTopology ();

      virtual SxPtr<SxTopology> clone () const;

      virtual void update (int nProcs_);

      // --- process distribution
      virtual bool isLocalTask (int iTask) const;
      virtual int getProc (int iTask) const;

   protected:
      int iProc;   // rank of process
      int nProcs;
      Type type;
      SxArray<int>  procMap;
};

#endif /* _SX_TOPOLOGY_H_ */
