#include <stdio.h>
#ifndef USE_MPI

int main ()
{
   printf ("Package was not compiled with MPI support.\n");
   return 0;
}

#else

#include <SxMPIMgr.h>
#include <SxMPI.h>
#include <SxThread.h>
#include <SxTopology.h>
#include <SxString.h>
#include <mpi.h>

class Writer : SxMPI
{
   public:
      Writer (const SxPtr<SxMPIMgr> &grp) 
         : SxMPI (grp, SxMPI::GroupMaster) { }
      int i;

      Writer (int i_ = 0) : i(i_) { }
      virtual ~Writer () { };

      virtual void main ()
      {
         i = 123;
         printf ("sending %d...\n", i);
         broadcast (i);
         printf ("THIS IS WRITER: %d\n", rank);
      }
};

class Reader : SxMPI
{
   public:
      int id;
      Reader (const SxPtr<SxMPIMgr> &grp, int id_) 
         : SxMPI (grp, SxMPI::GroupSlave), id(id_) { }
      virtual ~Reader () { }

      virtual void main ()
      {
         int i;
         i = broadcast<int> ();
         printf ("THIS IS READER id %d, received %d\n", id, i);
         fflush (stdout);
         SxThread::sleep (id);
      }
};


class SxPWTopology : public SxTopology
{
   public:
      SxPWTopology (int nSpin_, int nk_) 
         : SxTopology (),
           nSpin(nSpin_),
           nk(nk_)
      {
         dataMap.resize (nk);
         for (int ik=0; ik < nk; ++ik)  dataMap(ik).resize (nSpin);
         update (nProcs);
      }

      virtual ~SxPWTopology () { }

      virtual SxPtr<SxTopology> clone () const
      {
         SxPtr<SxPWTopology> res;
         res = SxPtr<SxPWTopology>::create (nSpin, nk);
         return res;
      }


      virtual void update (int nProcs_)
      {
         nProcs = nProcs_; 
         int rank = 0;
         for (int ik=0; ik < nk; ++ik)  {
            for (int iSpin=0; iSpin < nSpin; ++iSpin)  {
               dataMap(ik)(iSpin) = rank++;
               if (rank >= nProcs) rank = 0;
            }
         }
      }

      // --- process distribution
      virtual bool isLocalTask (int iTask) 
      {
         SX_EXIT;  // this is a SIMD topology!
         return true;
      }

      virtual int getProc (int iTask) const
      {
         SX_EXIT;  // this is a SIMD topology!
         return true;
      }

      // --- data distribution
      virtual bool isLocal (int iSpin, int ik)
      {
         return (iProc == dataMap(ik)(iSpin));
      }


// protected:
      int nSpin, nk;
      SxArray<SxArray<int> > dataMap;  // :ik,iSpin
};


int main (int argc, char **argv)
{
// SxPtr<SxMPIMgr> mgr = SxMPIMgr::getGlobalPtr ();
// SxPtr<SxMPIMgr> grp1 = mgr->addGroup ();
// SxPtr<SxMPIMgr> grp2 = mgr->addGroup ();
//
// // --- (1) client/server approach
// Writer writer (grp1);
// Reader r1(grp1, 1);
// Reader r2(grp1, 1);
// Reader r3(grp2, 5);
//
// mgr->print ();
//
// grp1->start ();
// grp1->barrier ();

   // --- (2) SPMD approach
   int nk=10, nSpin=2;
   SxPtr<SxPWTopology> topo = SxPtr<SxPWTopology>::create (nSpin, nk);
   for (int ik=0; ik < nk; ++ik)  {
      for (int iSpin=0; iSpin < nSpin; ++iSpin)  {
         if (topo->isLocal (iSpin,ik))  {
            printf ("%d: compute ik=%d, iSpin=%d\n", 
                    SxMPIMgr::getGlobalPtr()->getRank(), 
                    ik, iSpin);
         }
      }
   }
// printf ("back in main proc unit: %d\n", mgr->getRank ());
   return 0;
}

#endif
