#include <SxCLI.h>
#include <SxThreadPool.h>
#include <SxThread.h>

class MyThread : public SxThread
{
   public:
      SxString msg;
      MyThread (const SxString &msg);
      virtual ~MyThread ();
      virtual void main ();
};

MyThread::MyThread (const SxString &msg_)
   : msg(msg_)
{
   // empty
}

MyThread::~MyThread ()
{
   // empty
}

void MyThread::main ()
{
   cout << msg << endl;
}

int main (int argc, char **argv)
{
   SxCLI cli (argc, argv);
   int nThreads   = cli.option ("--nThreads",
                    "The number of system threads in threadpool."
                    ).toInt (1);
   cli.finalize ();
   
   // --- create thread pool
   SxThreadPool::initialize (nThreads);
   
   // --- SxThread will run automatically as a task in the thread pool
   MyThread a("a");
   a.start ();
   a.wait ();
   
   // --- submit another task to the thread pool manually
   SxPtr<MyThread> b = SxPtr<MyThread>::create("b");
   SxTask task(b);
   SxThreadPool::getGlobalObj().submit (task);
   b->wait ();
      
   return 0;
}
