#include <SxCLI.h>
#include <SxMarker.h>
#include <SxDir.h>
#include <SxFSAction.h>
#include <SxList.h>

SxString getBlock (const SxString &tmpl, const SxString &id)
{
   SxList<SxString> blocks = tmpl.getBlocks ("<!-- BEGIN " + id + " -->",
                                             "<!-- END " + id + " -->", false);
   SX_CHECK_VAR (blocks.getSize() == 1, blocks);
   return blocks.first ();
}

int main (int argc, char **argv)
{
   SxCLI cli (argc, argv);
   SxString cwd    = cli.option ("--cwd", "Path from where file list should be "
                                          "generated").toString (".");
   SxString projName = cli.option ("--name", "project name").toString ();
   SxString depth  = cli.option ("--depth", "Difference to top level tree").toString ();
   SxString topsrc = cli.option ("--srctop", "Top level of source tree").toString ();
   SxString accltop= cli.option ("--sxaccltop", "Top level of SxAccelerate").toString ("");
   SxString deps   = cli.option ("--deps", 
                                       "List of dependencies"
                                ).toString ("").substitute(" ",";").substitute(",", ";");
   SxString mode   = cli.option ("--mode", "Compilation mode").toString ();
   cli.finalize ();

   if (topsrc.contains ("/src"))  topsrc = depth + topsrc.right("/src");

   if (accltop == "")  accltop = "../" + topsrc;
   if (accltop.contains ("/src"))  accltop = depth + accltop.right("/src");

   SxList<SxString> modes;
   SxList<SxString>::ConstIterator mIt;

   SxString configType;
   if (mode == "withApps")  modes << "libOnly" << "withBin" << "withSBin" << "withLibExec";
   else                     modes << mode;

   for (mIt = modes.begin(); mIt != modes.end(); ++mIt)  {

      // --- dependencies
      SxList<SxString> tokens, depLibs, depIncs;
      tokens = deps.tokenize(';');
      SxList<SxString>::ConstIterator tIt;
      for (tIt = tokens.begin(); tIt != tokens.end(); ++tIt)  {
         SxString entry = tIt->substitute("_SolutionDir_", "$(SolutionDir)")
                              .substitute("_IntDir_",      "$(IntDir)")
                              .substitute("_OutDir_",      "$(OutDir)")
                              .substitute("/", "\\");

         if (entry.contains ("!"))  {
            // explicit CINCLUDE / LIBPATH specified
            depIncs << entry.left ("!");
            depLibs << entry.right ("!");
         }  else  {
            // external library?
            if (entry.contains (".lib") || entry.contains (".a"))  {
               depLibs << *tIt;
            }  else  {
               cout << entry; SX_EXIT;
               depIncs << ( entry );
               depLibs << ( entry + ".lib" );
            }
         }
      }
      depIncs << ( "$(SolutionDir)\\src\\windows" );

      SxString macros = "";
      if      (*mIt == "libOnly")      configType = "DynamicLibrary";
      else if (*mIt == "withBin")      configType = "Application";
      else if (*mIt == "withSBin")     configType = "Application";
      else if (*mIt == "withLibExec")  configType = "Application";
      else {
         cout << "ERROR: mode " << mode << " is not supported.\n";
         SX_QUIT;
      }

      if (mode == "withApps" && (*mIt == "withBin" || *mIt == "withSBin" || *mIt == "withLibExec"))  {
         macros = "SX_STANDALONE";
         depLibs.prepend ( projName + ".lib" );
      }


      try { SxFSAction::cd (cwd); } catch (SxException e) { e.print (); SX_QUIT; }

      SxString tmplPath = SxDir::getExecPath() + "/../system/sxtemplate.vcxproj";
      // support --with-bintarget
      if (!SxFSAction::test_f (tmplPath))
         tmplPath = SxDir::getExecPath() + "/../../system/sxtemplate.vcxproj";
      // fallback to source tree
      if (!SxFSAction::test_f (tmplPath))
         tmplPath = SxString(SXACCELERATE_SRC) + "/system/sxtemplate.vcxproj";
     
      if (!SxFSAction::test_f (tmplPath))
         SX_EXIT;


      SxString tmpl = SxString::read (tmplPath);

      SxString headerTmpl = getBlock (tmpl, "Header");
      SxString sourceTmpl = getBlock (tmpl, "Source");
      SxString lexTmpl    = getBlock (tmpl, "Lex");
      SxString yaccTmpl   = getBlock (tmpl, "Yacc");

      // --- inspect file structure
      SxList<SxString> headers, sources, lexes, yaccs;
      SxList<SxFileInfo> headerItems, sourceItems, lexItems, yaccItems;
      try { 
         headerItems = SxFSAction::ls ("*.h"); 
         sourceItems = SxFSAction::ls ("*.cpp"); 
         lexItems    = SxFSAction::ls ("*.lpp"); 
         yaccItems   = SxFSAction::ls ("*.ypp"); 
      }
      catch (SxException e)  { e.print (); SX_EXIT; }

      SxList<SxFileInfo>::ConstIterator fIt;

      if (configType == "DynamicLibrary")  {

         for (fIt = headerItems.begin(); fIt != headerItems.end(); ++fIt)
            headers << headerTmpl.substitute ("***Header.filename***", fIt->getName ());
         for (fIt = sourceItems.begin(); fIt != sourceItems.end(); ++fIt)
            sources << sourceTmpl.substitute ("***Source.filename***", fIt->getName ());
         for (fIt = lexItems.begin(); fIt != lexItems.end(); ++fIt)  {
            lexes << lexTmpl.substitute ("***Lex.filename***", fIt->getName ());
            SxString tabCpp = sourceTmpl.substitute ("***Source.filename***",
                                 fIt->getName().substitute (".lpp", ".tab.cpp"));
            if (!sources.contains (tabCpp))  sources << tabCpp;
         }
         for (fIt = yaccItems.begin(); fIt != yaccItems.end(); ++fIt)  {
            yaccs << yaccTmpl.substitute ("***Yacc.filename***", fIt->getName ());
            SxString yyCpp = sourceTmpl.substitute ("***Source.filename***",
                                fIt->getName().substitute (".ypp", ".yy.cpp"));
            if (!sources.contains (yyCpp))   sources << yyCpp;
         }

         // --- build blocks
         SxMap<SxString,SxString> blocks;
         blocks("Header") = SxString::join (headers, "\r\n");
         blocks("Source") = SxString::join (sources, "\r\n");
         blocks("Lex")    = SxString::join (lexes,   "\r\n");
         blocks("Yacc")   = SxString::join (yaccs,   "\r\n");
         tmpl = tmpl.substBlocks (blocks, true, "<!--", "-->");

         SxMarker marker;
         marker("NAME")        = projName;
         marker("DEPTH")       = depth.substitute("/", "\\");
         marker("TOPSRC")      = depth.substitute("/", "\\");
         marker("SXACCLTOP")   = accltop.substitute("/", "\\");
         marker("MACROS")      = macros;
         marker("DEPINCS")     = SxString::join (depIncs, ";").substitute("/", "\\");
         marker("DEPLIBS")     = SxString::join (depLibs, ";");
         marker("CONFIG_TYPE") = configType;
         marker("CONFIG_TYPE") = configType;

         SxString out = marker.apply (tmpl);

         // --- remove \r\n at the end
         ssize_t nChars = 0;
         if (out(out.getSize()-1)        == '\n')  nChars++;
         if (out(out.getSize()-nChars-1) == '\r')  nChars++;
         out = out(0,out.getSize()-nChars-1);

         out.write (cwd + "/" + projName + ".vcxproj");

      }  else  {

         for (fIt = sourceItems.begin(); fIt != sourceItems.end(); ++fIt)  {
            SxString project = fIt->getName ().substitute (".cpp", "");
            SxString srcName = sourceTmpl.substitute ("***Source.filename***", fIt->getName ());

            // --- build blocks
            SxMap<SxString,SxString> blocks;
            blocks("Header") = "";
            blocks("Source") = srcName;
            blocks("Lex")    = "";
            blocks("Yacc")   = "";

            SxMarker marker;
            marker("NAME")    = projName;
            marker("DEPTH")   = depth.substitute("/", "\\");
            marker("TOPSRC")  = depth.substitute("/", "\\");
            marker("SXACCLTOP") = accltop.substitute ("/", "\\");
            marker("MACROS")  = macros;
            marker("DEPINCS") = SxString::join (depIncs, ";").substitute("/", "\\");
            marker("DEPLIBS") = SxString::join (depLibs, ";");
            marker("CONFIG_TYPE")  = configType;

            SxString out = marker.apply (
                              tmpl.substBlocks (blocks, true, "<!--", "-->")
                           );

            // --- remove \r\n at the end
            ssize_t nChars = 0;
            if (out(out.getSize()-1)        == '\n')  nChars++;
            if (out(out.getSize()-nChars-1) == '\r')  nChars++;
            out = out(0,out.getSize()-nChars-1);
           
            out.write (cwd + "/" + project + ".vcxproj");
         }
      }
   }
   return 0;
}
