// ---------------------------------------------------------------------------
//
//      The ab-initio based multiscale library
//
//                  S / P H I / n X
//
//      Copyright:  Max-Planck-Institute for Iron Research
//                  40237 Duesseldorf, Germany
//
//      Contact:    https://sxlib.mpie.de
//      Authors:    see sphinx/AUTHORS
//      License:    see sphinx/LICENSE
//
// ---------------------------------------------------------------------------

#include <SxCalc.h>
#include <math.h>
int SxCalc_parse ();

int    SxCalc_pos = 0;
double SxCalc_res = 0.;
SxCalc *sxCalc = NULL;


SxCalc::SxCalc ()
{
   // empty
}


SxCalc::SxCalc (const SxArray<double> &vars_)
   : vars(vars_)
{
   // empty
}


SxCalc::~SxCalc ()
{
   // empty
}


double SxCalc::eval (const SxString &expr)
{
   sxCalc = this;
   buffer = expr;
   SxCalc_parseExpr (expr);
   return SxCalc_res;
}


SxString SxCalc::getExpr () const
{
   return buffer;
}


double SxCalc::getVar (int i) const
{
   if (i < 0 || i > vars.getSize())
      SxCalc_error ("Variable not defined.");

   return vars(i);

}

#ifdef WIN32
double SxCalc::cbrt (double x)
{
   return pow (x, 0.3333333333333333333333333);
}
#endif

// --------------------------------------------------------------------------


void SxCalc_error (const SxString &msg)
{
   cout << "ERROR: " << msg << endl;
   if (sxCalc)  {
      cout << sxCalc->getExpr() << endl;
      for (int i=0; i < SxCalc_pos-1; ++i) cout << " ";
      cout << "^\n";
   }
   SX_EXIT;
}
