// ---------------------------------------------------------------------------
//
//      The ab-initio based multiscale library
//
//                  S / P H I / n X
//
//      Copyright:  Max-Planck-Institute for Iron Research
//                  40237 Duesseldorf, Germany
//
//      Contact:    https://sxlib.mpie.de
//      Authors:    see sphinx/AUTHORS
//      License:    see sphinx/LICENSE
//
// ---------------------------------------------------------------------------

#include <SxCrashHandler.h>
#include <SxCrashListener.h>
#include <SxString.h>
#include <stdio.h>
#include <stdlib.h>

SxCrashHandler &SxCrashHandler::getGlobalInstance ()
{
   static SxCrashHandler handler;
   return handler;
}

SxCrashHandler::SxCrashHandler ()
{
   firstHandler = true;
}

SxCrashHandler::~SxCrashHandler ()
{
   // empty
}


void SxCrashHandler::commit (SxCrashListener *obj)
{
   listeners << obj;
}


void SxCrashHandler::deregister (SxCrashListener *obj)
{
   listeners.removeElement (obj);
}

void SxCrashHandler::trapCrash ()
{
   if (getGlobalInstance().firstHandler)  {
      // --- in case of many threads call this function only once
      getGlobalInstance().firstHandler = false;

      fflush (NULL); // flush all open streams

      if (getGlobalInstance().listeners.getSize() > 0)  {
         cout << endl << endl;
         cout << SEPARATOR;

         sxprintf ("| CRASH HANDLER: Commencing termination of program.\n");
         sxprintf ("|                Shutting down %d object(s).\n",
                 (int)getGlobalInstance().listeners.getSize());

         SxList<SxCrashListener *>::Iterator it;
         for (it  = getGlobalInstance().listeners.begin(); 
              it != getGlobalInstance().listeners.end(); 
              it++)  
         {
            (*it)->handleCrash ();
         }
         sxprintf ("| CRASH HANDLER: Shutdown sequence finished.\n\n");
         cout << SEPARATOR;
      }
   }
}
