// ---------------------------------------------------------------------------
//
//      The ab-initio based multiscale library
//
//                  S / P H I / n X
//
//      Copyright:  Max-Planck-Institute for Iron Research
//                  40237 Duesseldorf, Germany
//
//      Contact:    https://sxlib.mpie.de
//      Authors:    see sphinx/AUTHORS
//      License:    see sphinx/LICENSE
//
// ---------------------------------------------------------------------------

#ifndef _SX_HASH_TABLE_H_
#define _SX_HASH_TABLE_H_

#include <SxList.h>
#include <SxHashFunction.h>
#include <SxFunctor.h> /* SxNull */

/** \brief Hash table

    \b SxHashTable = SFHIngX Hash table

    \author Sixten Boeck, boeck@gemmantics.com
    \author Vaclav Bubnik, bubnik@gemmantics.com */
template <class K, class V, class H>
class SxHashTable : public SxArray<typename SxList<K>::List*>
{
   public:
      // --- Constructor 
      SxHashTable ();
      
      // -- Insertion
      bool contains (const K &, ssize_t *);
      inline typename SxList<V>::List* &value (ssize_t idx)
         { return values.elements[idx]; }
      inline typename SxList<V>::List* &value (ssize_t idx) const
         { return values.elements[idx]; }
      
      // -- Search
      ssize_t findPos (const K &) const;
      bool contains (const K &) const;
            
      // -- Deletion
      inline void remove (ssize_t);
      inline void removeAll ();
      
      void resize (ssize_t newSize);
      
      void print () const;
      
   protected:
      ssize_t tableSize;
      ssize_t tableUsed;
      ssize_t tableBound;
      
      SxArray<typename SxList<V>::List*> values;
      
      typename SxList<K>::List deleted;
};

/** \brief Hash table

    \b SxHashTable = SFHIngX Hash table

    \author Sixten Boeck, boeck@gemmantics.com
    \author Vaclav Bubnik, bubnik@gemmantics.com */
template <class K, class H>
class SxHashTable<K,SxNull,H> : public SxArray<typename SxList<K>::List*>
{
   public:
      // --- Constructor 
      SxHashTable ();
      
      // -- Insertion
      bool contains (const K &, ssize_t *);
      
      // -- Search
      ssize_t findPos (const K &) const;
      bool contains (const K &) const;
            
      // -- Deletion
      inline void remove (ssize_t);
      inline void removeAll ();
      
      void resize (ssize_t newSize);
      
      void print () const;
      
   protected:
      ssize_t tableSize;
      ssize_t tableUsed;
      ssize_t tableBound;
      
      typename SxList<K>::List deleted;
};

#include <SxHashTable.hpp>

#endif /* _SX_HASH_TABLE_H_ */
